@@ -90,6 +90,7 @@ from networking_bigswitch.plugins.bigswitch import constants as bsn_constants
 from networking_bigswitch.plugins.bigswitch.db import porttracker_db
 from networking_bigswitch.plugins.bigswitch import extensions
 from networking_bigswitch.plugins.bigswitch.i18n import _
+from networking_bigswitch.plugins.bigswitch.i18n import _LW
 from networking_bigswitch.plugins.bigswitch import servermanager
 from networking_bigswitch.plugins.bigswitch.utils import Util
 from networking_bigswitch.plugins.bigswitch import version
@@ -202,6 +203,10 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             True, if obj name, obj's tenant name and name have supported chars
             False, otherwise
         """
+
+        if self.servers.is_unicode_enabled():
+            return True
+
         if name and not servermanager.is_valid_bcf_name(name):
             LOG.warning('Unsupported characters in Name: %(name)s. ',
                         {'name': name})
@@ -280,7 +285,11 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                             [const.DEVICE_OWNER_ROUTER_GW,
                              const.DEVICE_OWNER_ROUTER_HA_INTF]):
                             continue
-                        mapped_port = self._map_tenant_name(port)
+                        mapped_port = self._map_display_name_or_tenant(port)
+                        if self.servers.is_unicode_enabled():
+                            # remove port name so that it won't be stored in
+                            #  description
+                            mapped_port['name'] = None
                         mapped_port = self._map_state_and_status(mapped_port)
                         mapped_port = self._map_port_hostid(mapped_port, net)
                         if not mapped_port:
@@ -326,7 +335,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                                 ext_tenant_id)
 
                     interfaces = []
-                    mapped_router = self._map_tenant_name(router)
+                    mapped_router = self._map_display_name_or_tenant(router)
                     mapped_router = self._map_state_and_status(mapped_router)
                     if not self._validate_names(mapped_router):
                         continue
@@ -340,8 +349,10 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                     for port in router_ports:
                         subnet_id = port['fixed_ips'][0]['subnet_id']
                         intf_details = self._get_router_intf_details(
-                            admin_context, subnet_id)
+                            admin_context, port, subnet_id)
+
                         interfaces.append(intf_details)
+
                     mapped_router['interfaces'] = interfaces
 
                     routers.append(mapped_router)
@@ -359,18 +370,22 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             # L3 plugin also includes tenant policies
             # data.update({'policies': tenant_policies})
 
-        if get_sgs and self.l3_plugin:
+        if (get_sgs and self.l3_plugin
+                and cfg.CONF.RESTPROXY.sync_security_groups):
             sgs = plugin.get_security_groups(admin_context) or []
             new_sgs = []
             for sg in sgs:
                 try:
-                    mapped_sg = self._map_tenant_name(sg)
+                    mapped_sg = self._map_display_name_or_tenant(sg)
                     if not self._validate_names(mapped_sg):
                         continue
                     if 'description' in mapped_sg:
                         mapped_sg['description'] = ''
-                    mapped_sg['name'] = Util.format_resource_name(
-                        mapped_sg['name'])
+                    if self.servers.is_unicode_enabled():
+                        mapped_sg['name'] = None
+                    else:
+                        mapped_sg['name'] = Util.format_resource_name(
+                            mapped_sg['name'])
                     new_sgs.append(mapped_sg)
                 except servermanager.TenantIDNotFound:
                     # if tenant name is not known to keystone, skip the sg
@@ -379,20 +394,35 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             data.update({'security-groups': new_sgs})
 
         all_tenants_map = self.servers.keystone_tenants
-        tenants = {}
-        for tenant in all_tenants_map:
-            if not self._validate_names(None, name=all_tenants_map[tenant]):
-                continue
-            tenants[tenant] = all_tenants_map[tenant]
 
-        data.update({'tenants': tenants})
+        if self.servers.is_unicode_enabled():
+            # display-name is only supported as list for topology in NSAPI
+            tenants = []
+            for tenant_id, tenant_name in all_tenants_map.items():
+                tenants.append({
+                    'name': tenant_id,
+                    'id': tenant_id,
+                    'display-name': tenant_name
+                })
+        else:
+            # dict for tenant works in topology sync only if display-name is
+            # not enabled
+            tenants = {}
+            for tenant in all_tenants_map:
+                if not self._validate_names(None,
+                                            name=all_tenants_map[tenant]):
+                    continue
+                tenants[tenant] = all_tenants_map[tenant]
+
+        data['tenants'] = tenants
         return data
 
     def _send_all_data_auto(self, timeout=None, triggered_by_tenant=None):
-        return self._send_all_data(send_floating_ips=self.l3_bsn_plugin,
-                                   send_routers=self.l3_bsn_plugin,
-                                   timeout=timeout,
-                                   triggered_by_tenant=triggered_by_tenant)
+        return self._send_all_data(
+            send_floating_ips=self.l3_bsn_plugin,
+            send_routers=self.l3_bsn_plugin,
+            timeout=timeout,
+            triggered_by_tenant=triggered_by_tenant)
 
     def _send_all_data(self, send_ports=True, send_floating_ips=True,
                        send_routers=True, send_sgs=True, timeout=None,
@@ -401,25 +431,21 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
 
         This gives the controller an option to re-sync it's persistent store
         with neutron's current view of that data.
-        """
-        data = self._get_all_data(
-            send_ports, send_floating_ips, send_routers, send_sgs)
-        # Lost keystone connection if data is None
-        # Log an error and continue
-        if data is None:
-            return None
 
-        data['triggered_by_tenant'] = triggered_by_tenant
-        errstr = _("Unable to update remote topology: %s")
-        return self.servers.rest_action('POST', servermanager.TOPOLOGY_PATH,
-                                        data, errstr, timeout=timeout)
+        All args are ignored. The `_get_all_data` method dynamically pulls the
+        relevant information i.e. if its L2 only or L2+L3.
+        """
+        sync_executed, topo_resp = self.servers.force_topo_sync()
+        return topo_resp
 
     def _assign_resource_to_service_tenant(self, resource):
         resource['tenant_id'] = (resource['tenant_id'] or
                                  servermanager.SERVICE_TENANT)
-        if resource.get('name'):
-            # resource name may contain space. Replace space with -
-            resource['name'] = Util.format_resource_name(resource['name'])
+
+        if not self.servers.is_unicode_enabled():
+            if resource.get('name'):
+                # resource name may contain space. Replace space with -
+                resource['name'] = Util.format_resource_name(resource['name'])
 
     def _get_network_with_floatingips(self, network, context=None):
         if context is None:
@@ -434,9 +460,11 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             for flip in fl_ips:
                 try:
                     # BVS-7525: the 'tenant_id' in a floating-ip represents the
-                    # tenant to which it is allocated. Validate that the
-                    # tenant exists
-                    mapped_flip = self._map_tenant_name(flip)
+                    # tenant to which it is allocated.
+                    # Validate that the tenant exists
+                    # name/display-name of floating ip is not actually
+                    # used on bcf
+                    mapped_flip = self._map_display_name_or_tenant(flip)
                     if mapped_flip.get('floating_port_id'):
                         fport = self.get_port(context,
                                               mapped_flip['floating_port_id'])
@@ -462,7 +490,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         if subnets:
             for subnet in subnets:
                 subnet_dict = self._make_subnet_dict(subnet, context=context)
-                mapped_subnet = self._map_tenant_name(subnet_dict)
+                mapped_subnet = self._map_display_name_or_tenant(subnet_dict)
                 mapped_subnet = self._map_state_and_status(mapped_subnet)
                 subnets_details.append(mapped_subnet)
 
@@ -474,31 +502,42 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         This network is not associated with any tenant
         """
         sg['tenant_id'] = sg['tenant_id'] or servermanager.SERVICE_TENANT
-        sg['tenant_name'] = self.servers.keystone_tenants.get(sg['tenant_id'])
-        if not sg['tenant_name']:
+        tenant_name = self.servers.keystone_tenants.get(sg['tenant_id'])
+
+        if not tenant_name:
             self.servers._update_tenant_cache(reconcile=True)
             tenant_name = self.servers.keystone_tenants.get(sg['tenant_id'])
+
+        if not self.servers.is_unicode_enabled():
             sg['tenant_name'] = tenant_name
+        return tenant_name
 
     def bsn_create_security_group(self, sg_id=None, sg=None, context=None):
         if sg_id:
+            if not hasattr(context, 'session'):
+                context = qcontext.get_admin_context()
             # overwrite sg if both sg and sg_id are given
             sg = self.get_security_group(context, sg_id)
 
         if sg:
-            sg['name'] = Util.format_resource_name(sg['name'])
+            if self.servers.is_unicode_enabled():
+                sg['display-name'] = sg['name']
+                sg['name'] = None
+            else:
+                sg['name'] = Util.format_resource_name(sg['name'])
             # remove description as its not used
-            if 'description' in sg:
-                sg['description'] = ''
-            self._tenant_check_for_security_group(sg)
+            if sg.get('description'):
+                del(sg['description'])
+            # check and map tenant_name for sg
+            tenant_name = self._tenant_check_for_security_group(sg)
             # skip the security group if its tenant is unknown
-            if sg['tenant_name']:
-                if sg['tenant_name'] == servermanager.SERVICE_TENANT:
+            if tenant_name:
+                if tenant_name == servermanager.SERVICE_TENANT:
                     self.bsn_create_tenant(servermanager.SERVICE_TENANT,
                                            context=context)
                 self.servers.rest_create_securitygroup(sg)
         else:
-            LOG.warning("No scurity group is provided for creation.")
+            LOG.warning(_LW("No security group is provided for creation."))
 
     def bsn_delete_security_group(self, sg_id, context=None):
         self.servers.rest_delete_securitygroup(sg_id)
@@ -510,14 +549,15 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         self.servers.rest_delete_tenant(tenant_id)
 
     def _verify_network_precommit(self, context):
-        if context.current['name'] != context.original['name']:
-            raise servermanager.NetworkNameChangeError()
+        if not self.servers.is_unicode_enabled():
+            if context.current['name'] != context.original['name']:
+                raise servermanager.NetworkNameChangeError()
 
     def _get_mapped_network_with_subnets(self, network, context=None):
         # if context is not provided, admin context is used
         if context is None:
             context = qcontext.get_admin_context()
-        network = self._map_tenant_name(network)
+        network = self._map_display_name_or_tenant(network)
         network = self._map_state_and_status(network)
         subnets = self._get_all_subnets_json_for_network(network['id'],
                                                          context)
@@ -536,6 +576,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         # OSP-45: remove name to avoid NSAPI error in convertToAscii
         for subnet in (subnets or []):
             subnet.pop('name', None)
+
         return network
 
     def _skip_bcf_network_event(self, network):
@@ -561,19 +602,22 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         if context is None:
             context = qcontext.get_admin_context()
         filters = {'name': ['default'], 'tenant_id': [tenant_id]}
-        default_group = self.get_security_groups(
-            context, filters, default_sg=True)
-        if default_group:
-            # VRRP tenant doesn't have tenant_id
-            self.bsn_create_security_group(sg=default_group[0])
+        if cfg.CONF.RESTPROXY.sync_security_groups:
+            default_group = self.get_security_groups(
+                context, filters, default_sg=True)
+            if default_group:
+                # VRRP tenant doesn't have tenant_id
+                self.bsn_create_security_group(sg=default_group[0])
+        # display-name is also mapped here
         mapped_network = self._get_mapped_network_with_subnets(network,
                                                                context)
 
         if not tenant_id:
             tenant_id = servermanager.SERVICE_TENANT
             mapped_network['tenant_id'] = servermanager.SERVICE_TENANT
-            mapped_network['name'] = Util.format_resource_name(
-                mapped_network['name'])
+            if not self.servers.is_unicode_enabled():
+                mapped_network['name'] = Util.format_resource_name(
+                    mapped_network['name'])
             self.bsn_create_tenant(servermanager.SERVICE_TENANT,
                                    context=context)
         self.servers.rest_create_network(tenant_id, mapped_network)
@@ -586,6 +630,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                      {'name': network.get('name')})
             return
 
+        # display-name is also mapped here
         mapped_network = self._get_mapped_network_with_subnets(network,
                                                                context)
         net_fl_ips = self._get_network_with_floatingips(mapped_network,
@@ -593,8 +638,9 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         if not tenant_id:
             tenant_id = servermanager.SERVICE_TENANT
             net_fl_ips['tenant_id'] = servermanager.SERVICE_TENANT
-            net_fl_ips['name'] = Util.format_resource_name(
-                net_fl_ips['name'])
+            if not self.servers.is_unicode_enabled():
+                net_fl_ips['name'] = Util.format_resource_name(
+                    net_fl_ips['name'])
         self.servers.rest_update_network(tenant_id, net_id, net_fl_ips)
 
     def _send_delete_network(self, network, context=None):
@@ -602,22 +648,35 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         tenant_id = network['tenant_id'] or servermanager.SERVICE_TENANT
         self.servers.rest_delete_network(tenant_id, net_id)
 
-    def _map_tenant_name(self, resource):
-        resource = copy.copy(resource)
+    def _map_display_name_or_tenant(self, resource):
+        """This maps tenant_name or display-name for an object
+
+        None-unicode mode uses tenant_name
+        Unicode mode uses tenant_id and display-name
+
+        :param resource: object to be mapped
+        :return: mapped object copy
+        """
+        resource = copy.deepcopy(resource)
         self._assign_resource_to_service_tenant(resource)
+
         tenant_name = self.servers.keystone_tenants.get(resource['tenant_id'])
-        if tenant_name:
-            resource['tenant_name'] = tenant_name
-        else:
+        if not tenant_name:
             self.servers._update_tenant_cache()
             tenant_name = self.servers.keystone_tenants.get(
                 resource['tenant_id'])
-            if tenant_name:
-                resource['tenant_name'] = tenant_name
-            else:
+            if not tenant_name:
                 raise servermanager.TenantIDNotFound(
                     tenant=resource['tenant_id'])
 
+        if self.servers.is_unicode_enabled():
+            if resource.get('name'):
+                resource['display-name'] = resource['name']
+            # cases like network needs the name on bcf side
+            resource['name'] = resource['id']
+        else:
+            resource['tenant_name'] = tenant_name
+
         return resource
 
     def _map_state_and_status(self, resource):
@@ -715,6 +774,26 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         bsn_host_id = port.get(portbindings.HOST_ID) + '-' + physnet
         return bsn_host_id
 
+    def _get_ovs_dpdk_port_hostid(self, port, network):
+        """Return HostID with bridge_name appended for OVS and DPDK ports
+
+        VIF type for OVS and DPDK ports is OVS and VHOSTUSER respectively.
+
+        :param port:
+        :param network:
+        :return: new_host_id
+        """
+        host_id = port.get(portbindings.HOST_ID)
+        physnet = network.get(pl_config.PROVIDER_PHYSNET)
+        if physnet not in self.bridge_mappings:
+            LOG.warning(_LW("Physical network to bridge mapping not "
+                            "found for port %s."), port)
+            return host_id
+
+        bridge_name = self.bridge_mappings.get(physnet)
+        host_id_bridge_name = host_id + '_' + bridge_name
+        return host_id_bridge_name
+
     def _map_port_hostid(self, port, network):
         """Update the HOST_ID of a given port based on it's type.
 
@@ -731,7 +810,6 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
 
         # Update HOST_ID (to be used by BCF).
         # - For SR-IOV it is a function of HostID & physnet info
-        # - For IVS and NFVSwitch, this is a param in vif_details
         if self._is_port_sriov(prepped_port):
             vif_type = prepped_port.get(portbindings.VIF_TYPE)
             if not vif_type or vif_type == portbindings.VIF_TYPE_UNBOUND:
@@ -742,12 +820,16 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             if not hostid:
                 return False
             prepped_port[portbindings.HOST_ID] = hostid
-        else:
-            vif_details = prepped_port.get(portbindings.VIF_DETAILS)
-            if vif_details:
-                hostid = vif_details.get(pl_config.VIF_DET_BSN_VSWITCH_HOST_ID)
-                if hostid:
-                    prepped_port[portbindings.HOST_ID] = hostid
+
+        # update HOST_ID to '<host-id>_<bridge-name>' for ports with
+        # VIF_TYPE OVS and VHOSTUSER i.e. DHCP and DPDK ports
+        # if bridge_name not available, sets it to just 'host-id'
+        vif_type = prepped_port.get(portbindings.VIF_TYPE)
+        if (vif_type and
+                (vif_type == portbindings.VIF_TYPE_OVS
+                 or vif_type == portbindings.VIF_TYPE_VHOST_USER)):
+            prepped_port[portbindings.HOST_ID] = (
+                self._get_ovs_dpdk_port_hostid(prepped_port, network))
 
         return prepped_port
 
@@ -763,14 +845,14 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                             "plugin. Ignoring setting status=%s.",
                             resource['status'])
 
-    def _get_router_intf_details(self, context, subnet_id):
+    def _get_router_intf_details(self, context, port, subnet_id):
 
         # we will use the network id as interface's id
         subnet = self.get_subnet(context, subnet_id)
         net_id = subnet['network_id']
         network = self.get_network(context, net_id)
         mapped_network = self._get_mapped_network_with_subnets(network)
-        mapped_subnet = self._map_tenant_name(subnet)
+        mapped_subnet = self._map_display_name_or_tenant(subnet)
         mapped_subnet = self._map_state_and_status(mapped_subnet)
 
         data = {
@@ -779,6 +861,10 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             "subnet": mapped_subnet
         }
 
+        # get gateway_ip from port instead of gateway_ip
+        if port.get("fixed_ips"):
+            data['ip_address'] = port["fixed_ips"][0]['ip_address']
+
         return data
 
     def _extend_port_dict_binding(self, context, port):
@@ -890,13 +976,12 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             raise lib_exceptions.PortNotFound(port_id=port_id)
 
 
-def put_context_in_serverpool(f):
+def add_debug_log(f):
     @functools.wraps(f)
     def wrapper(self, context, *args, **kwargs):
         # core plugin: context is top level object
-        # ml2: keeps context in _plugin_context
+        # Do not need context in header anymore
         LOG.debug("Function: %(fname)s called", {'fname': f.__name__})
-        self.servers.set_context(getattr(context, '_plugin_context', context))
         return f(self, context, *args, **kwargs)
     return wrapper
 
@@ -952,7 +1037,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         LOG.debug("NeutronRestProxyV2: initialization done")
 
     @db.context_manager.writer
-    @put_context_in_serverpool
+    @add_debug_log
     def create_network(self, context, network):
         """Create a network.
 
@@ -994,7 +1079,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         return new_net
 
     @db.context_manager.writer
-    @put_context_in_serverpool
+    @add_debug_log
     def update_network(self, context, net_id, network):
         """Updates the properties of a particular Virtual Network.
 
@@ -1029,7 +1114,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         return new_net
 
     @db.context_manager.writer
-    @put_context_in_serverpool
+    @add_debug_log
     def delete_network(self, context, net_id):
         """Delete a network.
 
@@ -1051,7 +1136,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         self._send_delete_network(orig_net, context)
         return ret_val
 
-    @put_context_in_serverpool
+    @add_debug_log
     def create_port(self, context, port):
         """Create a port.
 
@@ -1114,7 +1199,10 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
                 self._add_host_route(context, destination, new_port)
 
         # create on network ctrl
-        mapped_port = self._map_tenant_name(new_port)
+        mapped_port = self._map_display_name_or_tenant(new_port)
+        if self.servers.is_unicode_enabled():
+            # remove port name so that it won't be stored in description
+            mapped_port['name'] = None
         mapped_port = self._map_state_and_status(mapped_port)
         # ports have to be created synchronously when creating a router
         # port since adding router interfaces is a multi-call process
@@ -1143,7 +1231,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
                 self._extend_port_dict_binding(context, port)
         return [self._fields(port, fields) for port in ports]
 
-    @put_context_in_serverpool
+    @add_debug_log
     def update_port(self, context, port_id, port):
         """Update values of a port.
 
@@ -1204,7 +1292,11 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
                 # tenant_id must come from network in case network is shared
                 net_tenant_id = self._get_port_net_tenantid(context, new_port)
                 new_port = self._extend_port_dict_binding(context, new_port)
-                mapped_port = self._map_tenant_name(new_port)
+                mapped_port = self._map_display_name_or_tenant(new_port)
+                if self.servers.is_unicode_enabled():
+                    # remove port name so that it won't be stored in
+                    # description
+                    mapped_port['name'] = None
                 mapped_port = self._map_state_and_status(mapped_port)
                 self.servers.rest_update_port(net_tenant_id,
                                               new_port["network_id"],
@@ -1222,7 +1314,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
 
     # NOTE(kevinbenton): workaround for eventlet/mysql deadlock
     @runtime.synchronized('bsn-port-barrier')
-    @put_context_in_serverpool
+    @add_debug_log
     def delete_port(self, context, port_id, l3_port_check=True):
         """Delete a port.
 
@@ -1253,7 +1345,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
             # now that we've left db transaction, we are safe to notify
             self.l3_plugin.notify_routers_updated(context, router_ids)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def create_subnet(self, context, subnet):
         self._warn_on_state_status(subnet['subnet'])
 
@@ -1268,7 +1360,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
             self._send_update_network(orig_net, context)
         return new_subnet
 
-    @put_context_in_serverpool
+    @add_debug_log
     def update_subnet(self, context, id, subnet):
         self._warn_on_state_status(subnet['subnet'])
 
@@ -1283,7 +1375,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
             self._send_update_network(orig_net, context)
             return new_subnet
 
-    @put_context_in_serverpool
+    @add_debug_log
     def delete_subnet(self, context, id):
         orig_subnet = super(NeutronRestProxyV2, self).get_subnet(context, id)
         net_id = orig_subnet['network_id']
