@@ -32,7 +32,6 @@ import re
 import socket
 import ssl
 import time
-import weakref
 
 from neutron_lib import exceptions
 
@@ -45,6 +44,9 @@ from keystoneauth1 import session
 from keystoneclient.v3 import client as ksclient
 from networking_bigswitch.plugins.bigswitch.db import consistency_db as cdb
 from networking_bigswitch.plugins.bigswitch.i18n import _
+from networking_bigswitch.plugins.bigswitch.i18n import _LE
+from networking_bigswitch.plugins.bigswitch.i18n import _LI
+from networking_bigswitch.plugins.bigswitch.i18n import _LW
 from networking_bigswitch.plugins.bigswitch.utils import Util
 import os
 from oslo_config import cfg
@@ -85,7 +87,6 @@ FAILURE_CODES = [0, 301, 302, 303, 400, 401, 403, 404, 500, 501, 502, 503,
 BASE_URI = '/networkService/v2.0'
 ORCHESTRATION_SERVICE_ID = 'Neutron v2.0'
 HASH_MATCH_HEADER = 'X-BSN-BVS-HASH-MATCH'
-REQ_CONTEXT_HEADER = 'X-REQ-CONTEXT'
 SERVICE_TENANT = 'VRRP_Service'
 KS_AUTH_GROUP_NAME = 'keystone_authtoken'
 KS_AUTH_DOMAIN_DEFAULT = 'default'
@@ -95,6 +96,9 @@ HTTP_SERVICE_UNAVAILABLE_RETRY_COUNT = 3
 HTTP_SERVICE_UNAVAILABLE_RETRY_INTERVAL = 3
 
 KEYSTONE_SYNC_RATE_LIMIT = 30  # Limit KeyStone sync to once in 30 secs
+# TOPO_SYNC Responses
+TOPO_RESPONSE_OK = (httplib.OK, httplib.OK, True, True)
+TOPO_RESPONSE_FAIL = (0, None, None, None)
 
 # RE pattern for checking BCF supported names
 BCF_IDENTIFIER_UUID_RE = re.compile(r"[0-9a-zA-Z][-.0-9a-zA-Z_]*")
@@ -154,7 +158,7 @@ class NetworkNameChangeError(exceptions.NeutronException):
 
 
 class RemoteRestError(exceptions.NeutronException):
-    message = _("Error in REST call to remote network "
+    message = _("Error in REST call to BCF "
                 "controller: %(reason)s")
     status = None
 
@@ -252,6 +256,7 @@ class ServerProxy(object):
         self.name = name
         self.success_codes = SUCCESS_CODES
         self.auth = None
+        self.auth_token = None
         self.neutron_id = neutron_id
         self.failed = False
         self.capabilities = []
@@ -259,8 +264,13 @@ class ServerProxy(object):
         self.mypool = mypool
         # cache connection here to avoid a SSL handshake for every connection
         self.currentconn = None
+
         if auth:
-            self.auth = 'Basic ' + base64.encodestring(auth).strip()
+            if ':' in auth:
+                self.auth = 'Basic ' + base64.encodestring(auth).strip()
+            else:
+                self.auth_token = 'session_cookie="' + auth + '"'
+
         self.combined_cert = combined_cert
 
     def get_capabilities(self):
@@ -269,15 +279,15 @@ class ServerProxy(object):
             if body:
                 self.capabilities = jsonutils.loads(body)
         except Exception:
-            LOG.exception("Couldn't retrieve capabilities. "
-                          "Newer API calls won't be supported.")
+            LOG.exception("Couldn't retrieve capabilities on server "
+                          "%(server)s. ", {'server': self.server})
         LOG.info("The following capabilities were received "
                  "for %(server)s: %(cap)s",
                  {'server': self.server, 'cap': self.capabilities})
         return self.capabilities
 
     def rest_call(self, action, resource, data='', headers=None,
-                  timeout=False, reconnect=False, hash_handler=None):
+                  timeout=False, reconnect=False):
         uri = self.base_uri + resource
         body = jsonutils.dumps(data)
         headers = headers or {}
@@ -286,19 +296,6 @@ class ServerProxy(object):
         headers['NeutronProxy-Agent'] = self.name
         headers['Instance-ID'] = self.neutron_id
         headers['Orchestration-Service-ID'] = ORCHESTRATION_SERVICE_ID
-        lock_start_time = time.time()
-        if resource == TOPOLOGY_PATH:
-            # TopoSync call provides hash_handler but doesn't need hash checks
-            pass
-        elif hash_handler:
-            # For calls that need hash checks
-            headers[HASH_MATCH_HEADER] = hash_handler.read_for_update()
-        else:
-            # For calls that don't need hash checks CapabilityCheck
-            hash_handler = cdb.HashHandler()
-        lock_end_time = time.time()
-        LOG.debug("Time waited to get DB lock %.2fsecs",
-                  (lock_end_time - lock_start_time))
 
         # TODO(kevinbenton): Re-enable keep-alive in a thread-safe fashion.
         # When multiple workers are enabled the saved connection gets mangled
@@ -307,7 +304,10 @@ class ServerProxy(object):
             headers['Connection'] = 'keep-alive'
         else:
             reconnect = True
-        if self.auth:
+
+        if self.auth_token:
+            headers['Cookie'] = self.auth_token
+        elif self.auth:
             headers['Authorization'] = self.auth
 
         LOG.debug("ServerProxy: server=%(server)s, port=%(port)d, "
@@ -347,39 +347,20 @@ class ServerProxy(object):
                     return 0, None, None, None
 
         try:
+            bcf_request_time = time.time()
             currentconn.request(action, uri, body, headers)
             response = currentconn.getresponse()
             respstr = response.read()
             respdata = respstr
             bcf_response_time = time.time()
             LOG.debug("Time waited to get response from BCF %.2fsecs",
-                      (bcf_response_time - lock_end_time))
+                      (bcf_response_time - bcf_request_time))
             if response.status in self.success_codes:
-                hash_value = response.getheader(HASH_MATCH_HEADER)
-                if hash_value is not None:
-                    # There maybe be (rare) cases when the response takes too
-                    # long and thread looses the DB lock. In such cases
-                    # updating the HASH could result in chaos (eg: current
-                    # thread's lock is overwritten without mercy and would
-                    # allow another thread to grab the lock). Safest way
-                    # forward seems to be to check lock ownership prior to
-                    # updating Hash else let next REQUEST trigger a TopoSync
-                    if not hash_handler.put_hash_if_owner(hash_value):
-                        LOG.warning("Thread is no longer DB lock owner")
-                else:
-                    # Don't clear hash from DB if a hash header wasn't present
-                    hash_handler.clear_lock()
                 try:
                     respdata = jsonutils.loads(respstr)
                 except ValueError:
                     # response was not JSON, ignore the exception
                     pass
-            else:
-                # BVS-6979: race-condition(#2) on HashConflict, don't unlock
-                # to ensure topo_sync is scheduled next (it force grabs lock)
-                if response.status != httplib.CONFLICT:
-                    # release lock so others don't have to wait for timeout
-                    hash_handler.clear_lock()
 
             ret = (response.status, response.reason, respstr, respdata)
         except httplib.HTTPException:
@@ -430,6 +411,10 @@ class ServerPool(object):
         self.auth = cfg.CONF.RESTPROXY.server_auth
         self.ssl = cfg.CONF.RESTPROXY.server_ssl
         self.neutron_id = cfg.CONF.RESTPROXY.neutron_id
+        # unicode config
+        self.cfg_unicode_enabled = cfg.CONF.RESTPROXY.naming_scheme_unicode
+        self.capabilities = set()
+
         if 'keystone_authtoken' in cfg.CONF:
             self.auth_user = get_keystoneauth_cfg(cfg.CONF, 'username')
             self.auth_password = get_keystoneauth_cfg(cfg.CONF, 'password')
@@ -459,12 +444,11 @@ class ServerPool(object):
 
         self.base_uri = base_uri
         self.name = name
-        self.contexts = {}
         # Cache for Openstack projects
         # The cache is maintained in a separate thread and sync'ed with
         # Keystone periodically.
         self.keystone_tenants = {}
-        self._update_tenant_cache(reconcile=False)
+
         self.timeout = cfg.CONF.RESTPROXY.server_timeout
         self.always_reconnect = not cfg.CONF.RESTPROXY.cache_connections
         default_port = 8000
@@ -493,13 +477,23 @@ class ServerPool(object):
                 # strip [] for ipv6 address
                 server = server[1:-1]
             self.servers.append(self.server_proxy_for(server, int(port)))
+
+        # update capabilities
+        self.get_capabilities()
+
+        self._update_tenant_cache(reconcile=False)
+
         self.start_background_tasks()
+
         ServerPool._instance = self
+
         LOG.debug("ServerPool: initialization done")
 
     def start_background_tasks(self):
+        # consistency check, starts after 1* consistency_interval
         eventlet.spawn(self._consistency_watchdog,
                        cfg.CONF.RESTPROXY.consistency_interval)
+
         # Start keystone sync thread after 5 consistency sync
         # to give enough time for topology to sync over when
         # neutron-server starts.
@@ -508,37 +502,32 @@ class ServerPool(object):
             self._keystone_sync,
             cfg.CONF.RESTPROXY.keystone_sync_interval)
 
-    def set_context(self, context):
-        # this context needs to be local to the greenthread
-        # so concurrent requests don't use the wrong context.
-        # Use a weakref so the context is garbage collected
-        # after the plugin is done with it.
-        ref = weakref.ref(context)
-        self.contexts[eventlet.corolocal.get_ident()] = ref
-
-    def get_context_ref(self):
-        # Try to get the context cached for this thread. If one
-        # doesn't exist or if it's been garbage collected, this will
-        # just return None.
-        try:
-            return self.contexts[eventlet.corolocal.get_ident()]()
-        except KeyError:
-            return None
-
     def get_capabilities(self):
+        """Get capabilities
+
+        If cache has the value, use it
+        If Not, do REST calls to BCF controllers to check it
+
+        :return: supported capability list
+        """
         # lookup on first try
-        try:
+        # if capabilities is empty, the check is either not done, or failed
+        if self.capabilities:
             return self.capabilities
-        except AttributeError:
-            # this exception is hit when the capabilities haven't been
-            # looked up yet
-            pass
-        # each server should return a list of capabilities it supports
-        # e.g. ['floatingip']
-        capabilities = [set(server.get_capabilities())
-                        for server in self.servers]
-        # Pool only supports what all of the servers support
-        self.capabilities = set.intersection(*capabilities)
+
+        # Do REST calls to update capabilities at startup
+        # Servers should be the same version
+        # If one server is down, use online server's capabilities
+        capability_list = [set(server.get_capabilities())
+                           for server in self.servers]
+
+        new_capabilities = set.union(*capability_list)
+
+        if new_capabilities:
+            # Log unicode status
+            self.log_unicode_status(new_capabilities)
+            self.capabilities = new_capabilities
+
         # With multiple workers enabled, the fork may occur after the
         # connections to the DB have been established. We need to clear the
         # connections after the first attempt to call the backend to ensure
@@ -553,8 +542,50 @@ class ServerPool(object):
         # ec716b9e68b8b66a88218913ae4c9aa3a26b025a/neutron/wsgi.py#L104
         if cdb.HashHandler._FACADE:
             cdb.HashHandler._FACADE.get_engine().pool.dispose()
+
+        if not new_capabilities:
+            LOG.error('Failed to get capabilities on any controller. ')
         return self.capabilities
 
+    def log_unicode_status(self, new_capabilities):
+        """Log unicode running status
+
+        :param new_capabilities: new capabilities
+        :return:
+        """
+        # unicode disabled by user
+        if not self.cfg_unicode_enabled:
+            LOG.info('naming_scheme_unicode is set to False,'
+                     ' Unicode names Disabled')
+        # unicode enabled and supported by controller
+        elif 'display-name' in new_capabilities:
+            LOG.info('naming_scheme_unicode is set to True,'
+                     ' Unicode names Enabled')
+        # unicode enabled, but not supported by controller
+        else:
+            LOG.warning('naming_scheme_unicode is set to True,'
+                        ' but BCF does not support it.'
+                        ' Unicode names Disabled')
+
+    def is_unicode_enabled(self):
+        """Check unicode running status
+
+        True: enabled
+        False: disabled
+        """
+        LOG.debug('Current Capabilities: %s', self.get_capabilities())
+        if not self.get_capabilities():
+            msg = 'Capabilities unknown! Please check BCF controller status.'
+            raise RemoteRestError(reason=msg)
+
+        if self.cfg_unicode_enabled and 'display-name' in \
+                self.get_capabilities():
+            LOG.debug('Unicode Check=True')
+            return True
+        else:
+            LOG.debug('Unicode Check=False')
+            return False
+
     def server_proxy_for(self, server, port):
         combined_cert = self._get_combined_cert_for_server(server, port)
         return ServerProxy(server, port, self.ssl, self.auth, self.neutron_id,
@@ -630,7 +661,7 @@ class ServerPool(object):
         """
         try:
             cert = ssl.get_server_certificate((server, port),
-                                              ssl_version=ssl.PROTOCOL_TLSv1)
+                                              ssl_version=ssl.PROTOCOL_SSLv23)
         except Exception as e:
             raise cfg.Error(_('Could not retrieve initial '
                               'certificate from controller %(server)s. '
@@ -666,32 +697,8 @@ class ServerPool(object):
         """
         return resp[0] in SUCCESS_CODES
 
-    def dblock_mark_toposync_started(self, orig_hash_handler):
-        """Insert DBLock with TopoSync start marker
-
-        marker = 'TOPO' + original call's lockID
-        @return: new HashHandler for TopoSync
-        """
-        prefix = cdb.DBLOCK_PREFIX_TOPO + orig_hash_handler.random_lock_id
-        handler = cdb.HashHandler(prefix=prefix)
-        new = handler.lock_marker + "initial:hash,code"
-        handler.put_hash(new)
-        return handler
-
     def rest_call(self, action, resource, data, headers, ignore_codes,
                   timeout=False):
-        context = self.get_context_ref()
-        if context:
-            # include the requesting context information if available
-            cdict = context.to_dict()
-            # remove the auth token so it's not present in debug logs on the
-            # backend controller
-            cdict.pop('auth_token', None)
-            if ('tenant_name' in cdict and cdict['tenant_name']):
-                cdict['tenant_name'] = Util.format_resource_name(
-                    cdict['tenant_name'])
-            headers[REQ_CONTEXT_HEADER] = jsonutils.dumps(cdict)
-        hash_handler = cdb.HashHandler()
         good_first = sorted(self.servers, key=lambda x: x.failed)
         first_response = None
         for active_server in good_first:
@@ -704,43 +711,10 @@ class ServerPool(object):
             for x in range(HTTP_SERVICE_UNAVAILABLE_RETRY_COUNT + 1):
                 ret = active_server.rest_call(action, resource, data, headers,
                                               timeout,
-                                              reconnect=self.always_reconnect,
-                                              hash_handler=hash_handler)
+                                              reconnect=self.always_reconnect)
                 if ret[0] != httplib.SERVICE_UNAVAILABLE:
                     break
-                time.sleep(HTTP_SERVICE_UNAVAILABLE_RETRY_INTERVAL)
-
-            # If inconsistent, do a full synchronization
-            if ret[0] == httplib.CONFLICT and hash_handler.is_db_lock_owner():
-                if not self.get_topo_function:
-                    raise cfg.Error(_('Server requires synchronization, '
-                                      'but no topology function was defined.'))
-
-                LOG.info("ServerProxy: HashConflict detected with request "
-                         "%(action)s %(resource)s Starting Topology sync",
-                         {'action': action, 'resource': resource})
-                topo_hh = self.dblock_mark_toposync_started(hash_handler)
-                try:
-                    data = self.get_topo_function(
-                        **self.get_topo_function_args)
-                    if data:
-                        ret_ts = active_server.rest_call('POST', TOPOLOGY_PATH,
-                                                         data, timeout=None,
-                                                         hash_handler=topo_hh)
-                        if self.server_failure(ret_ts, ignore_codes):
-                            LOG.error("ServerProxy: Topology sync failed")
-                            raise RemoteRestError(reason=ret_ts[2],
-                                                  status=ret_ts[0])
-                finally:
-                    LOG.info("ServerProxy: Topology sync completed")
-                    if data is None:
-                        return None
-            elif ret[0] == httplib.CONFLICT and \
-                    not hash_handler.is_db_lock_owner():
-                # DB lock ownership lost, allow current owner to detect hash
-                # conflict and perform needed TopoSync
-                LOG.warning("HashConflict detected but thread is no longer"
-                            " DB lock owner. Skipping TopoSync call")
+                eventlet.sleep(HTTP_SERVICE_UNAVAILABLE_RETRY_INTERVAL)
 
             # Store the first response as the error to be bubbled up to the
             # user since it was a good server. Subsequent servers will most
@@ -771,13 +745,6 @@ class ServerPool(object):
                              'ret': ret[2], 'data': ret[3]})
                 active_server.failed = True
 
-        # A failure on a delete means the object is gone from Neutron but not
-        # from the controller. Set the consistency hash to a bad value to
-        # trigger a sync on the next check.
-        # NOTE: The hash must have a comma in it otherwise it will be ignored
-        # by the backend.
-        if action == 'DELETE':
-            hash_handler.put_hash('INCONSISTENT,INCONSISTENT')
         # All servers failed, reset server list and try again next time
         LOG.error('ServerProxy: %(action)s failure for all servers: '
                   '%(server)r',
@@ -802,6 +769,18 @@ class ServerPool(object):
         resp = self.rest_call(action, resource, data, headers, ignore_codes,
                               timeout)
         if self.server_failure(resp, ignore_codes):
+            # Request wasn't success, nor can be ignored,
+            # do a full synchronization if auto_sync_on_failure is True
+            if (cfg.CONF.RESTPROXY.auto_sync_on_failure and
+                    resource != TOPOLOGY_PATH):
+                LOG.error(_LE("NeutronRestProxyV2: Inconsistency with backend "
+                              "controller, triggering full synchronization. "
+                              "%(action)s %(resource)s."),
+                          {'action': action, 'resource': resource})
+                sync_executed, topo_resp = self.force_topo_sync(check_ts=True)
+                if sync_executed:
+                    return topo_resp
+            # either topo_sync not executed or topo_sync itself failed
             LOG.error(errstr, resp[2])
             raise RemoteRestError(reason=resp[2], status=resp[0])
         if resp[0] in ignore_codes:
@@ -843,13 +822,16 @@ class ServerPool(object):
         if not tenant_name:
             raise TenantIDNotFound(tenant=tenant_id)
 
-        if not is_valid_bcf_name(tenant_name):
-            raise UnsupportedNameException(obj_type=ObjTypeEnum.tenant,
-                                           obj_id=tenant_id,
-                                           obj_name=tenant_name)
-
+        if self.is_unicode_enabled():
+            data = {"tenant_id": tenant_id, 'tenant_name': tenant_id,
+                    'display-name': tenant_name}
+        else:
+            if not is_valid_bcf_name(tenant_name):
+                raise UnsupportedNameException(obj_type=ObjTypeEnum.tenant,
+                                               obj_id=tenant_id,
+                                               obj_name=tenant_name)
+            data = {"tenant_id": tenant_id, 'tenant_name': tenant_name}
         resource = TENANT_RESOURCE_PATH
-        data = {"tenant_id": tenant_id, 'tenant_name': tenant_name}
         errstr = _("Unable to create tenant: %s")
         self.rest_action('POST', resource, data, errstr)
 
@@ -1022,7 +1004,7 @@ class ServerPool(object):
     def _consistency_watchdog(self, polling_interval=60):
         if 'consistency' not in self.get_capabilities():
             LOG.warning("Backend server(s) do not support automated "
-                        "consitency checks.")
+                        "consistency checks.")
             return
         if not polling_interval:
             LOG.warning("Consistency watchdog disabled by polling "
@@ -1040,6 +1022,79 @@ class ServerPool(object):
                 LOG.exception("Encountered an error checking controller "
                               "health.")
 
+    def force_topo_sync(self, check_ts=True):
+        """Execute a topology_sync between OSP and BCF.
+
+        Topology sync collects all data from Openstack and pushes to BCF in
+        one single REST call. This is a heavy operation and is not executed
+        automatically.
+
+        Conditions when this would be called:
+        (1) during ServerPool initialization
+            (a) must check previous timestamp
+        (2) if periodic keystone tenant_cache find a diff in tenant list
+            (a) should not check previous timestamp
+        (3) externally triggered by neutron force-bcf-sync command
+            (a) should not check previous timestamp
+        (4) a rest_call to BCF fails on both servers and failure_code is not
+            part of the ignore_codes list
+            (a) must check previous timestamp
+
+        :param check_ts: boolean flag to check previous
+                         timestamp < TOPO_SYNC_EXPIRED_SECS
+               prev_resp: a REST response tuple from the previous failed REST
+                          call if available. If we skip topo_sync, return the
+                          failure as previously observed.
+        :return: (sync_executed, response)
+                 sync_executed - Boolean - returns True if we were able to
+                                acquire a lock to perform topo_sync,
+                                False otherwise
+                 response - tuple of the typical HTTP response from REST call
+                        (response.status, response.reason, respstr, respdata)
+        """
+        LOG.info(_LI('TOPO_SYNC requested with check_ts %s'), check_ts)
+
+        if not self.get_topo_function:
+            raise cfg.Error(_('Server requires synchronization, '
+                              'but no topology function was defined.'))
+
+        # get current timestamp
+        curr_ts = str(time.time())
+        hash_handler = cdb.HashHandler(timestamp_ms=curr_ts)
+
+        if not hash_handler.lock(check_ts):
+            LOG.info(_LI("TOPO_SYNC: lock() returned False. Skipping."))
+            return False, TOPO_RESPONSE_OK
+
+        # else, perform topo_sync
+        try:
+            LOG.debug("TOPO_SYNC: requested at %(request_ts)s started at "
+                      "%(start_ts)s",
+                      {'request_ts': cdb.convert_ts_to_datetime(curr_ts),
+                       'start_ts': cdb.convert_ts_to_datetime(time.time())})
+            data = self.get_topo_function(
+                **self.get_topo_function_args)
+            if not data:
+                # when keystone sync fails, it fails silently with data = None
+                # that is wrong, we need to raise an exception
+                raise Exception(_("TOPO_SYNC: failed to retrieve data."))
+            LOG.debug("TOPO_SYNC: data received from OSP, sending "
+                      "request to BCF.")
+            errstr = _("Unable to perform forced topology_sync: %s")
+            return True, self.rest_action('POST', TOPOLOGY_PATH, data, errstr)
+        except Exception as e:
+            # if encountered an exception, set to previous timestamp
+            LOG.warning(_LW("TOPO_SYNC: Exception during topology sync. "
+                            "Consistency DB timestamp will not be updated."))
+            hash_handler.unlock(set_prev_ts=True)
+            raise e
+        finally:
+            hash_handler.unlock()
+            diff = time.time() - float(hash_handler.lock_ts)
+            LOG.info(_LI("TOPO_SYNC: took %s seconds to execute topo_sync. "
+                         "consistency_db unlocked."),
+                     str(diff))
+
     def _ensure_tenant_cache(self, tenant_id):
         if tenant_id not in self.keystone_tenants:
             self._update_tenant_cache()
@@ -1060,8 +1115,13 @@ class ServerPool(object):
             sess = session.Session(auth=auth)
             keystone_client = ksclient.Client(session=sess)
             tenants = keystone_client.projects.list()
-            new_cached_tenants = {tn.id: Util.format_resource_name(tn.name)
-                                  for tn in tenants}
+
+            if self.is_unicode_enabled():
+                new_cached_tenants = {tn.id: tn.name
+                                      for tn in tenants}
+            else:
+                new_cached_tenants = {tn.id: Util.format_resource_name(tn.name)
+                                      for tn in tenants}
             # Add SERVICE_TENANT to handle hidden network for VRRP
             new_cached_tenants[SERVICE_TENANT] = SERVICE_TENANT
 
@@ -1078,15 +1138,8 @@ class ServerPool(object):
                     LOG.debug("TENANT delete: id %s", tenant_id)
                     self.rest_delete_tenant(tenant_id)
                 if diff.changed():
-                    hash_handler = cdb.HashHandler()
-                    res = hash_handler._get_current_record()
-                    if res:
-                        lock_owner = hash_handler._get_lock_owner(res.hash)
-                        if lock_owner and cdb.DBLOCK_PREFIX_TOPO in lock_owner:
-                            # topology sync is still going on
-                            return True
-                    LOG.debug("TENANT changed: force topo sync")
-                    hash_handler.put_hash('initial:hash,code')
+                    LOG.debug("Tenant cache outdated. Forcing topo_sync.")
+                    self.force_topo_sync(check_ts=False)
             return True
         except Exception:
             LOG.exception("Encountered an error syncing with keystone.")
@@ -1117,8 +1170,8 @@ class HTTPSConnectionWithValidation(httplib.HTTPSConnection):
             self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
                                         cert_reqs=ssl.CERT_REQUIRED,
                                         ca_certs=self.combined_cert,
-                                        ssl_version=ssl.PROTOCOL_TLSv1)
+                                        ssl_version=ssl.PROTOCOL_SSLv23)
         else:
             self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
                                         cert_reqs=ssl.CERT_NONE,
-                                        ssl_version=ssl.PROTOCOL_TLSv1)
+                                        ssl_version=ssl.PROTOCOL_SSLv23)
