@@ -15,6 +15,7 @@
 import copy
 import datetime
 import httplib
+import os
 
 import eventlet
 from oslo_config import cfg
@@ -24,6 +25,7 @@ from oslo_utils import excutils
 from oslo_utils import timeutils
 
 from neutron.common import rpc as n_rpc
+from neutron.extensions import securitygroup as ext_sg
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -34,25 +36,69 @@ from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
 
 from networking_bigswitch.plugins.bigswitch import config as pl_config
-from networking_bigswitch.plugins.bigswitch.db import consistency_db as cdb
 from networking_bigswitch.plugins.bigswitch.i18n import _
+from networking_bigswitch.plugins.bigswitch.i18n import _LE
+from networking_bigswitch.plugins.bigswitch.i18n import _LI
+from networking_bigswitch.plugins.bigswitch.i18n import _LW
 from networking_bigswitch.plugins.bigswitch import plugin
 from networking_bigswitch.plugins.bigswitch import servermanager
 
-from networking_bigswitch.plugins.bigswitch.config import VHOST_USER_SOCKET_DIR
-from networking_bigswitch.plugins.bigswitch.config \
-    import VIF_DET_BSN_VSWITCH_HOST_ID
-from networking_bigswitch.plugins.bigswitch.config import VSwitchType
-
 EXTERNAL_PORT_OWNER = 'neutron:external_port'
 ROUTER_GATEWAY_PORT_OWNER = 'network:router_gateway'
+OVS_AGENT_INI_FILEPATH = '/etc/neutron/plugins/ml2/openvswitch_agent.ini'
+RH_NET_CONF_PATH = "/etc/os-net-config/config.json"
 LOG = log.getLogger(__name__)
-put_context_in_serverpool = plugin.put_context_in_serverpool
+add_debug_log = plugin.add_debug_log
 
 # time in seconds to maintain existence of vswitch response
 CACHE_VSWITCH_TIME = 60
 
 
+def _read_ovs_bridge_mappings():
+    """Read the 'bridge_mappings' property from openvswitch_agent.ini
+
+    This is done for Redhat environments, to allow an improved
+    learning/programming of interface groups based on ports and the network
+    to which the ports belong to.
+
+    :return: bridge_mappings dictionary {'physnet_name': 'bridge_name', ...}
+                {} empty dictionary when not found
+    """
+    mapping = {}
+    mapping_str = None
+    # read openvswitch_agent.ini for bridge_mapping info
+    if not os.path.isfile(OVS_AGENT_INI_FILEPATH):
+        # if ovs_agent.ini doesn't exists, return empty mapping
+        LOG.warning(_LW("Unable to read OVS bridge_mappings, "
+                        "openvswitch_agent.ini file not present."))
+        return mapping
+
+    with open(OVS_AGENT_INI_FILEPATH) as f:
+        for line in f:
+            if ('#' not in line and
+                    ('=' in line and 'bridge_mappings' in line)):
+                # typical config line looks like the following:
+                # bridge_mappings = datacentre:br-ex,dpdk:br-link
+                key, value = line.split('=', 1)
+                mapping_str = value.strip()
+
+    # parse comma separated physnet list into individual mappings
+    if not mapping_str:
+        # if file did not have bridge_mappings, return empty mapping
+        LOG.warning(_LW(
+            "Unable to read OVS bridge_mappings, either the line is commented "
+            "or not present in openvswitch_agent.ini."))
+        return mapping
+
+    phy_map_list = mapping_str.split(',')
+    for phy_map in phy_map_list:
+        phy, bridge = phy_map.split(':')
+        mapping[phy.strip()] = bridge.strip()
+
+    LOG.info(_LI("OVS bridge_mappings are: %(br_map)s"), {'br_map': mapping})
+    return mapping
+
+
 class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
                                api.MechanismDriver):
 
@@ -70,39 +116,57 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
         pl_config.register_config()
         self.evpool = eventlet.GreenPool(cfg.CONF.RESTPROXY.thread_pool_size)
 
-        hash_handler = cdb.HashHandler()
-        if hash_handler.is_db_hash_empty():
-            LOG.debug("Forcing topology sync as consistency hash is empty")
-            hash_handler.read_for_update()
-            hash_handler.put_hash('initial:hash,code')
-
         # init network ctrl connections
         self.servers = servermanager.ServerPool()
-        self.servers.get_topo_function = self._get_all_data_auto
+        self.servers.get_topo_function = self._get_all_data
+        self.servers.get_topo_function_args = {'get_ports': True,
+                                               'get_floating_ips': True,
+                                               'get_routers': True,
+                                               'get_sgs': True}
+        # perform one forced topo_sync after 60secs. delayed to let plugin
+        # initialization complete
+        eventlet.spawn_after(60, self.servers.force_topo_sync,
+                             **{'check_ts': True})
+
         self.segmentation_types = ', '.join(cfg.CONF.ml2.type_drivers)
+        # if os-net-config is present, attempt to read physnet bridge_mappings
+        # from openvswitch_agent.ini
+        self.bridge_mappings = {}
+        if os.path.isfile(RH_NET_CONF_PATH):
+            self.bridge_mappings = _read_ovs_bridge_mappings()
         # Track hosts running IVS to avoid excessive calls to the backend
-        self.vswitch_host_cache = {}
-        self.setup_sg_rpc_callbacks()
+        self.ivs_host_cache = {}
+        self.setup_rpc_callbacks()
 
         LOG.debug("Initialization done")
 
-    def setup_sg_rpc_callbacks(self):
-        # following way to register call back functions start in kilo
-        self._create_sg_f = self.bsn_create_sg_callback
-        self._delete_sg_f = self.bsn_delete_sg_callback
-        self._update_sg_f = self.bsn_update_sg_callback
-        self._create_sg_rule_f = self.bsn_create_sg_rule_callback
-        self._delete_sg_rule_f = self.bsn_delete_sg_rule_callback
-        registry.subscribe(self._create_sg_f,
-                           resources.SECURITY_GROUP, events.AFTER_CREATE)
-        registry.subscribe(self._delete_sg_f,
-                           resources.SECURITY_GROUP, events.AFTER_DELETE)
-        registry.subscribe(self._update_sg_f,
-                           resources.SECURITY_GROUP, events.AFTER_UPDATE)
-        registry.subscribe(self._create_sg_rule_f,
-                           resources.SECURITY_GROUP_RULE, events.AFTER_CREATE)
-        registry.subscribe(self._delete_sg_rule_f,
-                           resources.SECURITY_GROUP_RULE, events.AFTER_DELETE)
+    def setup_rpc_callbacks(self):
+        # Security group operations are split between callback and RPC
+        # notifications mechanism.
+        # Callbacks are 'only once' and RPC notifications are 'at least once'.
+        # In a HA setup, RPC notifications can be called once per overcloud
+        # controller.
+        # Callback: SG add, SG delete, SG update, SG rule add
+        # RPC notifications: SG add, SG delete, SG rule delete
+        # In addition, RPC notifications also used for: tenant add,
+        # tenant delete, log all other events as FYI.
+
+        # Register callbacks ONLY IF sync_security_groups is True
+        if cfg.CONF.RESTPROXY.sync_security_groups:
+            # following way to register call back functions start in kilo
+            self._create_sg_f = self.bsn_create_sg_callback
+            self._delete_sg_f = self.bsn_delete_sg_callback
+            self._update_sg_f = self.bsn_update_sg_callback
+            self._create_sg_rule_f = self.bsn_create_sg_rule_callback
+            registry.subscribe(self._create_sg_f,
+                               resources.SECURITY_GROUP, events.AFTER_CREATE)
+            registry.subscribe(self._delete_sg_f,
+                               resources.SECURITY_GROUP, events.AFTER_DELETE)
+            registry.subscribe(self._update_sg_f,
+                               resources.SECURITY_GROUP, events.AFTER_UPDATE)
+            registry.subscribe(self._create_sg_rule_f,
+                               resources.SECURITY_GROUP_RULE,
+                               events.AFTER_CREATE)
 
         # the above does not cover the cases where security groups are
         # initially created or when they are deleted since those actions
@@ -152,38 +216,56 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             LOG.debug("Callback create rule in sg_id: %s", sg_id)
             self.bsn_create_security_group(sg_id=sg_id, context=context)
 
-    def bsn_delete_sg_rule_callback(self, resource, event, trigger, **kwargs):
-        context = kwargs.get('context')
-        if context:
-            LOG.debug("Callback deleted sg_rule belones to tenant: %s",
-                      context.tenant_id)
-            sgs = self.get_security_groups(context, filters={}) or []
-            for sg in sgs:
-                if sg.get('tenant_id') != context.tenant_id:
-                    continue
-                sg_id = sg.get('id')
-                LOG.debug("Callback delete rule in sg_id: %s", sg_id)
-                # we over write the sg on bcf controller instead of deleting
-                self.bsn_create_security_group(sg_id=sg_id, context=context)
+    def bsn_delete_sg_rule(self, sg_rule, context):
+        LOG.debug("Deleting security group rule from BCF: %s", sg_rule)
+        if not context:
+            LOG.error(_LE(
+                "Context missing when trying to delete security group rule. "
+                "Please force-bcf-sync to ensure consistency with BCF."))
+        sg_id = sg_rule['security_group_id']
+        # we over write the sg on bcf controller instead of deleting
+        try:
+            self.bsn_create_security_group(sg_id,
+                                           context=context)
+        except ext_sg.SecurityGroupNotFound:
+            # DB query will throw exception when security group is
+            # being deleted. delete_security_group_rule callback would
+            # try to update BCF with new set of rules.
+            LOG.warning(
+                _LW("Security group with ID %(sg_id)s not found "
+                    "when trying to update."), {'sg_id': sg_id})
 
     def info(self, ctxt, publisher_id, event_type, payload, metadata):
         """This is called on each notification to the neutron topic """
+        # we retain this section for security groups, because it handles
+        # other events as well. Ignore security group events if disabled in
+        # config
         if event_type == 'security_group.create.end':
             LOG.debug("Security group created: %s", payload)
-            self.bsn_create_security_group(sg=payload['security_group'])
+            if cfg.CONF.RESTPROXY.sync_security_groups:
+                self.bsn_create_security_group(sg=payload['security_group'])
         elif event_type == 'security_group.delete.end':
             LOG.debug("Security group deleted: %s", payload)
-            self.bsn_delete_security_group(payload['security_group_id'])
+            if cfg.CONF.RESTPROXY.sync_security_groups:
+                self.bsn_delete_security_group(payload['security_group_id'])
+        elif event_type == 'security_group_rule.delete.end':
+            LOG.debug("Security group rule deleted: %s", payload)
+            if cfg.CONF.RESTPROXY.sync_security_groups:
+                self.bsn_delete_sg_rule(payload['security_group_rule'], ctxt)
         elif event_type == 'identity.project.deleted':
             LOG.debug("Project deleted: %s", payload)
             self.bsn_delete_tenant(payload['resource_info'])
         elif event_type == 'identity.project.created':
             LOG.debug("Project created: %s", payload)
             self.bsn_create_tenant(payload['resource_info'])
+        elif event_type == 'identity.project.updated':
+            LOG.debug("Project updated: %s", payload)
+            # update is the same as create, nsapi will handle it
+            self.bsn_create_tenant(payload['resource_info'])
         else:
             LOG.debug("Else events: %s payload: %s", (event_type, payload))
 
-    @put_context_in_serverpool
+    @add_debug_log
     def security_groups_rule_updated(self, context, **kwargs):
         # this will get called whenever a security group rule updated message
         # goes onto the RPC bus
@@ -192,37 +274,37 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             for sg_id in kwargs.get('security_groups'):
                 self.bsn_create_security_group(sg_id, context=context)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def security_groups_member_updated(self, context, **kwargs):
         pass
 
-    @put_context_in_serverpool
+    @add_debug_log
     def security_groups_provider_updated(self, context, **kwargs):
         pass
 
-    @put_context_in_serverpool
+    @add_debug_log
     def create_network_postcommit(self, context):
         # create network on the network controller
         self._send_create_network(context.current)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def update_network_precommit(self, context):
         self._verify_network_precommit(context)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def update_network_postcommit(self, context):
         # update network on the network controller
         self._send_update_network(context.current)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def update_subnet_postcommit(self, context):
         self._trigger_network_update_from_subnet_transaction(context)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def create_subnet_postcommit(self, context):
         self._trigger_network_update_from_subnet_transaction(context)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def delete_subnet_postcommit(self, context):
         self._trigger_network_update_from_subnet_transaction(context)
 
@@ -231,12 +313,12 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             ctx.get_admin_context(), context.current['network_id'])
         self._send_update_network(net)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def delete_network_postcommit(self, context):
         # delete network on the network controller
         self._send_delete_network(context.current)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def create_port_postcommit(self, context):
         if not self._is_port_supported(context.current):
             LOG.debug("Ignoring unsupported vnic type")
@@ -270,7 +352,7 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             self.async_port_create(port["network"]["tenant_id"],
                                    port["network"]["id"], port)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def update_port_postcommit(self, context):
         if not self._is_port_supported(context.current):
             LOG.debug("Ignoring unsupported vnic type")
@@ -300,11 +382,6 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             return
 
         if port:
-            # For vhostuser type ports, membership rule and endpoint was
-            # created during bind_port, so skip this
-            if port[portbindings.VIF_TYPE] == portbindings.VIF_TYPE_VHOST_USER:
-                return
-
             try:
                 # For SR-IOV ports, we shouldn't update the port status
                 update_status = not self._is_port_sriov(port)
@@ -323,7 +400,7 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
                             triggered_by_tenant=port["network"]["tenant_id"]
                         )
 
-    @put_context_in_serverpool
+    @add_debug_log
     def delete_port_postcommit(self, context):
         if not self._is_port_supported(context.current):
             LOG.debug("Ignoring unsupported vnic type")
@@ -334,7 +411,7 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
         net = context.network.current
         tenant_id = net['tenant_id']
         if not tenant_id:
-            tenant_id = plugin.SERVICE_TENANT
+            tenant_id = servermanager.SERVICE_TENANT
         self.servers.rest_delete_port(tenant_id, net["id"], port['id'])
 
     def _prepare_port_for_controller(self, context):
@@ -346,91 +423,26 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
         net = context.network.current
         port['network'] = net
         port['bound_segment'] = context.top_bound_segment
-        prepped_port = self._map_tenant_name(port)
+        prepped_port = self._map_display_name_or_tenant(port)
+        if prepped_port.get('description'):
+            del (prepped_port['description'])
+        if self.servers.is_unicode_enabled():
+            prepped_port['name'] = None
         prepped_port = self._map_state_and_status(prepped_port)
         prepped_port = self._map_port_hostid(prepped_port, net)
         return prepped_port
 
-    def _bind_port_nfvswitch(self, context, segment, host_id):
-        """Perform bind_port for nfvswitch.
-
-        A NFV VM needs to be attached to a nfv-switch socket. So, during
-        bind_port() we create a NFV VM endpoint on BCF, thereby reserving the
-        socket for it's use. Then pass the sock_path in the set_binding() for
-        Nova to plug the VM to the nfv-switch.
-
-        @param context: PortContext object
-        """
-        vif_type = portbindings.VIF_TYPE_VHOST_USER
-        port = self._prepare_port_for_controller(context)
-        if not port:
-            LOG.warning("nfv-switch bind_port() skipped due to missing "
-                        "Host ID.")
-            return
-
-        # Create an endpoint corresponding to the port on the Controller,
-        # thereby asking the Controller to reserve a vhost_sock for it
-        tenant_id = port["network"]["tenant_id"]
-        network_id = port["network"]["id"]
-        # Set vif_type to 'vhost_user' for the Controller to reserve vhost_sock
-        port[portbindings.VIF_TYPE] = vif_type
-        # Update host_id so that endpoint create will have the correct value
-        port[portbindings.HOST_ID] = host_id
-        try:
-            self.async_port_create(tenant_id, network_id, port)
-        except servermanager.RemoteRestError as e:
-            with excutils.save_and_reraise_exception() as ctxt:
-                if (cfg.CONF.RESTPROXY.auto_sync_on_failure and
-                        e.status == httplib.NOT_FOUND and
-                        servermanager.NXNETWORK in e.reason):
-                    ctxt.reraise = False
-                    LOG.error("Inconsistency with backend controller "
-                              "triggering full synchronization.")
-                    self._send_all_data_auto(triggered_by_tenant=tenant_id)
-
-        # Retrieve the vhost_socket reserved for the port(endpoint) by the
-        # Controller and use it in set_binding()
-        resp = self.servers.rest_get_port(tenant_id, network_id, port["id"])
-        if not resp or not isinstance(resp, list):
-            LOG.warning("Controller failed to reserve a nfv-switch sock")
-            return
-
-        vhost_sock = None
-        attachment_point = resp[0].get('attachment-point')
-        if attachment_point:
-            vhost_sock = attachment_point.get('interface')
-
-        if not vhost_sock:
-            LOG.warning("Controller failed to reserve a nfv-switch sock")
-            return
-
-        vhost_sock_path = self._get_vhost_user_sock_path(vhost_sock)
-        LOG.info('nfv-switch VM %(port)s alloted sock_path %(sock)s',
-                 {'port': port['id'], 'sock': vhost_sock_path})
-
-        # Update vif_details with host_id. This way, for all BCF
-        # communications, we we shall use it as HOST_ID (i.e. interface-group
-        # on BCF)
-        vif_details = {portbindings.CAP_PORT_FILTER: False,
-                       portbindings.VHOST_USER_MODE:
-                       portbindings.VHOST_USER_MODE_SERVER,
-                       portbindings.VHOST_USER_OVS_PLUG: False,
-                       portbindings.VHOST_USER_SOCKET: vhost_sock_path,
-                       VIF_DET_BSN_VSWITCH_HOST_ID: host_id}
-        context.set_binding(segment[api.ID], vif_type, vif_details)
-
-    def _bind_port_ivswitch(self, context, segment, host_id):
+    def _bind_port_ivswitch(self, context, segment):
         """Perform bind_port for Indigo virtual switch.
 
         @param context: PortContext object
         """
         vif_type = pl_config.VIF_TYPE_IVS
         vif_details = {portbindings.CAP_PORT_FILTER: True,
-                       portbindings.OVS_HYBRID_PLUG: True,
-                       VIF_DET_BSN_VSWITCH_HOST_ID: host_id}
+                       portbindings.OVS_HYBRID_PLUG: True}
         context.set_binding(segment[api.ID], vif_type, vif_details)
 
-    @put_context_in_serverpool
+    @add_debug_log
     def bind_port(self, context):
         """Marks ports as bound.
 
@@ -458,51 +470,18 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             LOG.debug("SR-IOV port, nothing to do")
             return
 
-        # A compute node can have multiple vswitches. They are differentiated
-        # on BCF based on the lldps sent by them.
-        #   IVS shall be identified as 'HOST'
-        #   Others (eg. NFVSwitch) shall be identified as 'HOST'.'PHYSNET'
-        #
-        # Check each segment to identify the correct vswitch, and set it in
-        # vif_details
-        for segment in context.segments_to_bind:
-            if segment[api.NETWORK_TYPE] == const.TYPE_VLAN:
-                (vswitch_type, host_id) = self.get_vswitch_type(context.host,
-                                                                segment)
-                if vswitch_type == VSwitchType.NFVSWITCH:
-                    self._bind_port_nfvswitch(context, segment, host_id)
-                elif vswitch_type == VSwitchType.VIRTUAL:
-                    self._bind_port_ivswitch(context, segment, host_id)
+        # IVS hosts will have a vswitch with the same name as the hostname
+        if self.does_vswitch_exist(context.host):
+            for segment in context.segments_to_bind:
+                if segment[api.NETWORK_TYPE] == const.TYPE_VLAN:
+                    self._bind_port_ivswitch(context, segment)
 
-    def get_vswitch_type(self, host, segment=None):
-        """Get the virtual switch type on the given host for the given segment.
+    def does_vswitch_exist(self, host):
+        """Check if Indigo vswitch exists with the given hostname.
 
-        Check for virtual switch on host.physnet, else check on host.
-
-        @param host: the HOST_ID.
-        @param segment: the segment.
-        @returns: (vswitch-type, host-id): if vswitch is found on the host.
-                  (None, _): otherwise.
-        """
-        if segment and segment[api.PHYSICAL_NETWORK]:
-            physnet = segment[api.PHYSICAL_NETWORK]
-            host_physnet = host + "." + physnet
-            host_physnet_vswitch_type = self._get_vswitch_type(host_physnet)
-            if host_physnet_vswitch_type:
-                return (host_physnet_vswitch_type, host_physnet)
-
-        return (self._get_vswitch_type(host), host)
-
-    def _get_vswitch_type(self, host):
-        """Get virtual switch type
-
-        Check if a virtual switch exists with the given hostname on BCF, if
-        it does, return its type.
-
-        @param host: the HOST_ID.
-        @returns: switch-type, if the vswitch is known to BCF.
-                  None, if vswitch is unknown to BCF or backend couldn't be
-                  reached.
+        Returns True if switch exists on backend.
+        Returns False if switch does not exist.
+        Returns None if backend could not be reached.
         Caches response from backend.
         """
         try:
@@ -511,43 +490,30 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             # cache was empty for that switch or expired
             pass
 
-        switch_type = None
         try:
-            resp = self.servers.rest_get_switch(host)
-            exists = bool(resp)
-            if exists:
-                switch_type = resp[0]["fabric-role"]
+            exists = bool(self.servers.rest_get_switch(host))
         except servermanager.RemoteRestError:
             # Connectivity or internal server error. Skip cache to try again on
             # next binding attempt
             return
-
-        self.vswitch_host_cache[host] = {
-            'type': switch_type,
+        self.ivs_host_cache[host] = {
             'timestamp': datetime.datetime.now(),
             'exists': exists
         }
-        return switch_type
+        return exists
 
     def _get_cached_vswitch_existence(self, host):
         """Returns cached existence.
 
         Expired and non-cached raise ValueError.
         """
-        entry = self.vswitch_host_cache.get(host)
+        entry = self.ivs_host_cache.get(host)
         if not entry:
             raise ValueError(_('No cache entry for host %s') % host)
 
         diff = timeutils.delta_seconds(entry['timestamp'],
                                        datetime.datetime.now())
         if diff > CACHE_VSWITCH_TIME:
-            self.vswitch_host_cache.pop(host)
+            self.ivs_host_cache.pop(host)
             raise ValueError(_('Expired cache entry for host %s') % host)
-
-        if entry['exists']:
-            return entry['type']
-        return None
-
-    def _get_vhost_user_sock_path(self, sock):
-        """Get the socket path for the vhost_user socket. """
-        return VHOST_USER_SOCKET_DIR + str(sock)
+        return entry['exists']
