@@ -28,6 +28,9 @@ from neutron_lib import context
 from neutron_lib.plugins import directory
 
 from networking_bigswitch.plugins.bigswitch import config as pl_config
+from networking_bigswitch.plugins.bigswitch import constants as bsn_constants
+from networking_bigswitch.plugins.bigswitch.servermanager import\
+    TenantIDNotFound
 from networking_bigswitch.tests.unit.bigswitch import fake_server
 from networking_bigswitch.tests.unit.bigswitch \
     import test_base as bsn_test_base
@@ -35,6 +38,8 @@ from networking_bigswitch.tests.unit.bigswitch \
 patch = mock.patch
 HTTPCON = ('networking_bigswitch.plugins.bigswitch.servermanager.httplib'
            '.HTTPConnection')
+IS_UNICODE_ENABLED = ('networking_bigswitch.plugins.bigswitch.servermanager.'
+                      'ServerPool.is_unicode_enabled')
 
 
 class BigSwitchProxyPluginV2TestCase(bsn_test_base.BigSwitchTestBase,
@@ -48,12 +53,13 @@ class BigSwitchProxyPluginV2TestCase(bsn_test_base.BigSwitchTestBase,
         self.setup_patches()
         if plugin_name:
             self._plugin_name = plugin_name
-        l3_plugin = {'L3_ROUTER_NAT': self._l3_plugin_name}
+        bsn_service_plugins = {'L3_ROUTER_NAT': self._l3_plugin_name,
+                               bsn_constants.BSN_SERVICE_PLUGIN:
+                                   self._bsn_service_plugin_name}
         if service_plugins:
-            service_plugins.update(l3_plugin)
+            service_plugins.update(bsn_service_plugins)
         else:
-            service_plugins = l3_plugin
-
+            service_plugins = bsn_service_plugins
         super(BigSwitchProxyPluginV2TestCase,
               self).setUp(self._plugin_name,
                           service_plugins=service_plugins,
@@ -245,22 +251,6 @@ class TestBigSwitchProxyPortsV2(test_plugin.TestPortsV2,
         with self.port(**kwargs) as port:
             self.assertEqual(port['port']['binding:vif_type'], self.VIF_TYPE)
 
-    def test_port_nfvswitch_vif_details_override(self):
-        # nfvhost is in the test config to override to vhostuser
-        kwargs = {'name': 'name', 'binding:host_id': 'nfvhost',
-                  'device_id': 'override_dev',
-                  'arg_list': ('binding:host_id',)}
-        with self.port(**kwargs) as port:
-            self.assertEqual(port['port']['binding:vif_type'],
-                             portbindings.VIF_TYPE_VHOST_USER)
-
-        self._delete('ports', port['port']['id'])
-        self._delete('networks', port['port']['network_id'])
-        kwargs = {'name': 'name2', 'binding:host_id': 'someotherhost',
-                  'device_id': 'other_dev'}
-        with self.port(**kwargs) as port:
-            self.assertEqual(port['port']['binding:vif_type'], self.VIF_TYPE)
-
     def test_port_move(self):
         # ivshost is in the test config to override to IVS
         kwargs = {'name': 'name', 'binding:host_id': 'ivshost',
@@ -354,6 +344,109 @@ class TestBigSwitchProxySync(BigSwitchProxyPluginV2TestCase):
         self.assertEqual(result[0], 200)
 
 
+class TestDisplayName(BigSwitchProxyPluginV2TestCase):
+    def get_true(self):
+        """Used for side_effect replacement
+
+        :return:
+        """
+        return True
+
+    def test_map_display_name_or_tenant_unicode_disabled(self):
+        """Test _map_display_name_or_tenant behaviors when unicode is disabled
+
+        :return:
+        """
+        self.map_display_name_or_tenant_p.stop()
+        plugin_obj = directory.get_plugin()
+
+        self.assertFalse(plugin_obj.servers.is_unicode_enabled())
+
+        # object with non-existing tenant_id
+        no_tenant_obj = {'id': 'test_id',
+                         'name': 'test_name',
+                         'tenant_id': 'non_exist_tenant_id'}
+
+        self.assertRaises(TenantIDNotFound,
+                          plugin_obj._map_display_name_or_tenant,
+                          no_tenant_obj)
+
+        # add a tenant to cache
+        plugin_obj.servers.keystone_tenants = {'tenant_id': 'tenant_name'}
+
+        # object with name, '_' in name will be replaced with '__'
+        test_obj = {'id': 'test_id',
+                    'name': 'test_name',
+                    'tenant_id': 'tenant_id'}
+
+        expected_obj = {'id': 'test_id',
+                        'name': 'test__name',
+                        'tenant_id': 'tenant_id',
+                        'tenant_name': 'tenant_name'}
+
+        self.assertEqual(expected_obj,
+                         plugin_obj._map_display_name_or_tenant(test_obj))
+
+        # object without name
+        test_obj = {'id': 'test_id',
+                    'tenant_id': 'tenant_id'}
+
+        expected_obj = {'id': 'test_id',
+                        'tenant_id': 'tenant_id',
+                        'tenant_name': 'tenant_name'}
+
+        self.assertEqual(expected_obj,
+                         plugin_obj._map_display_name_or_tenant(test_obj))
+
+    def test_map_display_name_or_tenant_unicode_enabled(self):
+        """Test _map_display_name_or_tenant behaviors when unicode is enabled
+
+        :return:
+        """
+        self.map_display_name_or_tenant_p.stop()
+        self.is_unicode_enabled_p.stop()
+        mock.patch(IS_UNICODE_ENABLED, side_effect=self.get_true).start()
+        plugin_obj = directory.get_plugin()
+
+        self.assertTrue(plugin_obj.servers.is_unicode_enabled())
+
+        # object with non-existing tenant_id, unicode enabled
+        no_tenant_obj = {'id': 'test_id',
+                         'name': 'test_name',
+                         'tenant_id': 'non_exist_tenant_id'}
+
+        self.assertRaises(TenantIDNotFound,
+                          plugin_obj._map_display_name_or_tenant,
+                          no_tenant_obj)
+
+        # add a tenant to cache
+        plugin_obj.servers.keystone_tenants = {'tenant_id': 'tenant_name'}
+
+        # object with name, unicode enabled
+        test_obj = {'id': 'test_id',
+                    'name': 'test_name',
+                    'tenant_id': 'tenant_id'}
+
+        expected_obj = {'id': 'test_id',
+                        'name': 'test_id',
+                        'display-name': 'test_name',
+                        'tenant_id': 'tenant_id'}
+
+        self.assertEqual(expected_obj,
+                         plugin_obj._map_display_name_or_tenant(test_obj))
+
+        # object without name, unicode enabled
+        test_obj = {'id': 'test_id',
+                    'tenant_id': 'tenant_id'}
+
+        expected_obj = {'id': 'test_id',
+                        'name': 'test_id',
+                        'tenant_id': 'tenant_id'}
+
+        self.assertEqual(expected_obj,
+                         plugin_obj._map_display_name_or_tenant(test_obj))
+
+
 class TestBigSwitchAddressPairs(test_addr_pair.TestAllowedAddressPairs,
                                 BigSwitchProxyPluginV2TestCase):
     def test_create_missing_mac_field(self):
