@@ -113,9 +113,9 @@ class RouterDBTestCase(RouterDBTestBase,
             router_req = self.new_create_request('routers', data, self.fmt)
             res = router_req.get_response(self.ext_api)
             router = self.deserialize(self.fmt, res)
-            self.assertEqual(
-                s['subnet']['tenant_id'],
-                router['router']['external_gateway_info']['tenant_id'])
+            # external tenant_id should be patched
+            self.assertTrue(
+                'tenant_id' in router['router']['external_gateway_info'])
 
     def test_router_add_interface_port(self):
         orig_update_port = self.plugin.update_port
