@@ -14,24 +14,24 @@
 import httplib
 import socket
 import ssl
+import time
 
 import mock
 from oslo_config import cfg
 from oslo_db import exception as db_exc
-from oslo_serialization import jsonutils
 from oslo_utils import importutils
 
 from networking_bigswitch.plugins.bigswitch.db import consistency_db
 from networking_bigswitch.plugins.bigswitch import servermanager
 from networking_bigswitch.tests.unit.bigswitch \
     import test_restproxy_plugin as test_rp
-from neutron_lib import context
 from neutron_lib.plugins import directory
 
 SERVERMANAGER = 'networking_bigswitch.plugins.bigswitch.servermanager'
 CONSISTENCYDB = 'networking_bigswitch.plugins.bigswitch.db.consistency_db'
 HTTPCON = SERVERMANAGER + '.httplib.HTTPConnection'
 HTTPSCON = SERVERMANAGER + '.HTTPSConnectionWithValidation'
+SERVER_GET_CAPABILITIES = SERVERMANAGER + '.ServerPool.get_capabilities'
 
 
 class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
@@ -75,7 +75,7 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
                 *('example.org', 443)
             )
             sslgetmock.assert_has_calls([mock.call(
-                ('example.org', 443), ssl_version=ssl.PROTOCOL_TLSv1)])
+                ('example.org', 443), ssl_version=ssl.PROTOCOL_SSLv23)])
 
     def test_consistency_watchdog_stops_with_0_polling_interval(self):
         pl = directory.get_plugin()
@@ -88,8 +88,9 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
 
     def test_consistency_watchdog(self):
         pl = directory.get_plugin()
-        pl.servers.capabilities = []
+        pl.servers.capabilities = ['dummy']
         self.watch_p.stop()
+
         with mock.patch('eventlet.sleep') as smock,\
                 mock.patch(
                     SERVERMANAGER + '.ServerPool.rest_call',
@@ -103,58 +104,13 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
             # should return immediately without consistency capability
             pl.servers._consistency_watchdog()
             self.assertFalse(smock.called)
+
             pl.servers.capabilities = ['consistency']
             self.assertRaises(KeyError,
                               pl.servers._consistency_watchdog)
             rmock.assert_called_with('GET', '/health', '', {}, [], False)
             self.assertEqual(1, len(lmock.mock_calls))
 
-    def test_consistency_hash_header(self):
-        # mock HTTP class instead of rest_call so we can see headers
-        with mock.patch(HTTPCON) as conmock:
-            rv = conmock.return_value
-            rv.getresponse.return_value.getheader.return_value = 'HASHHEADER'
-            rv.getresponse.return_value.status = 200
-            rv.getresponse.return_value.read.return_value = ''
-            with self.network() as network:
-                callheaders = rv.request.mock_calls[0][1][3]
-                self.assertIn('X-BSN-BVS-HASH-MATCH', callheaders)
-                # first call will be empty to indicate no previous state hash
-                self.assertEqual(callheaders['X-BSN-BVS-HASH-MATCH'], '')
-                # change the header that will be received on delete call
-                rv.getresponse.return_value.getheader.return_value = 'HASH2'
-            self._delete('networks', network['network']['id'])
-            # net delete should have used header received on create
-            callheaders = rv.request.mock_calls[1][1][3]
-            self.assertEqual(callheaders['X-BSN-BVS-HASH-MATCH'], 'HASHHEADER')
-
-            # create again should now use header received from prev delete
-            with self.network():
-                callheaders = rv.request.mock_calls[2][1][3]
-                self.assertIn('X-BSN-BVS-HASH-MATCH', callheaders)
-                self.assertEqual(callheaders['X-BSN-BVS-HASH-MATCH'],
-                                 'HASH2')
-
-    def test_consistency_hash_header_no_update_on_bad_response(self):
-        # mock HTTP class instead of rest_call so we can see headers
-        with mock.patch(HTTPCON) as conmock:
-            rv = conmock.return_value
-            rv.getresponse.return_value.getheader.return_value = 'HASHHEADER'
-            rv.getresponse.return_value.status = 200
-            rv.getresponse.return_value.read.return_value = ''
-            with self.network() as net:
-                # change the header that will be received on delete call
-                rv.getresponse.return_value.getheader.return_value = 'EVIL'
-                rv.getresponse.return_value.status = 'GARBAGE'
-                self._delete('networks', net['network']['id'])
-
-            # create again should not use header from delete call
-            with self.network():
-                callheaders = rv.request.mock_calls[2][1][3]
-                self.assertIn('X-BSN-BVS-HASH-MATCH', callheaders)
-                self.assertEqual(callheaders['X-BSN-BVS-HASH-MATCH'],
-                                 'HASHHEADER')
-
     def test_file_put_contents(self):
         pl = directory.get_plugin()
         with mock.patch(SERVERMANAGER + '.open', create=True) as omock:
@@ -184,6 +140,7 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
                 mock.call.write('certdata')
             ])
 
+    # basic authentication
     def test_auth_header(self):
         cfg.CONF.set_override('server_auth', 'username:pass', 'RESTPROXY')
         sp = servermanager.ServerPool()
@@ -193,9 +150,23 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
             sp.rest_create_network('tenant', 'network')
         callheaders = rv.request.mock_calls[0][1][3]
         self.assertIn('Authorization', callheaders)
+        self.assertNotIn('Cookie', callheaders)
         self.assertEqual(callheaders['Authorization'],
                          'Basic dXNlcm5hbWU6cGFzcw==')
 
+    # token based authentication
+    def test_auth_token_header(self):
+        cfg.CONF.set_override('server_auth', 'fake_token', 'RESTPROXY')
+        sp = servermanager.ServerPool()
+        with mock.patch(HTTPCON) as conmock:
+            rv = conmock.return_value
+            rv.getresponse.return_value.getheader.return_value = 'HASHHEADER'
+            sp.rest_create_network('tenant', 'network')
+        callheaders = rv.request.mock_calls[0][1][3]
+        self.assertIn('Cookie', callheaders)
+        self.assertNotIn('Authorization', callheaders)
+        self.assertEqual(callheaders['Cookie'], 'session_cookie="fake_token"')
+
     def test_header_add(self):
         sp = servermanager.ServerPool()
         with mock.patch(HTTPCON) as conmock:
@@ -211,23 +182,6 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
         self.assertIn('EXTRA-HEADER', callheaders)
         self.assertEqual(callheaders['EXTRA-HEADER'], 'HI')
 
-    def test_req_context_header(self):
-        sp = directory.get_plugin().servers
-        ncontext = context.Context('uid', 'tid')
-        sp.set_context(ncontext)
-        with mock.patch(HTTPCON) as conmock:
-            rv = conmock.return_value
-            rv.getresponse.return_value.getheader.return_value = 'HASHHEADER'
-            sp.rest_action('GET', '/')
-        callheaders = rv.request.mock_calls[0][1][3]
-        self.assertIn(servermanager.REQ_CONTEXT_HEADER, callheaders)
-        ctxdct = ncontext.to_dict()
-        # auth token is not included
-        ctxdct.pop('auth_token')
-        self.assertEqual(
-            ctxdct, jsonutils.loads(
-                callheaders[servermanager.REQ_CONTEXT_HEADER]))
-
     def test_capabilities_retrieval(self):
         sp = servermanager.ServerPool()
         with mock.patch(HTTPCON) as conmock:
@@ -236,14 +190,18 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
 
             # each server will get different capabilities
             rv.read.side_effect = ['["a","b","c"]', '["b","c","d"]']
-            # pool capabilities is intersection between both
-            self.assertEqual(set(['b', 'c']), sp.get_capabilities())
+            # pool capabilities is union of both
+            # normally capabilities should be the same across all servers
+            # this only happens in two situations:
+            # 1. a server is down
+            # 2. during upgrade/downgrade
+            self.assertEqual(set(['a', 'b', 'c', 'd']), sp.get_capabilities())
             self.assertEqual(2, rv.read.call_count)
 
-            # the pool should cache after the first call so no more
-            # HTTP calls should be made
+            # the pool should cache after the first call
+            # so no more HTTP calls should be made
             rv.read.side_effect = ['["w","x","y"]', '["x","y","z"]']
-            self.assertEqual(set(['b', 'c']), sp.get_capabilities())
+            self.assertEqual(set(['a', 'b', 'c', 'd']), sp.get_capabilities())
             self.assertEqual(2, rv.read.call_count)
 
     def test_capabilities_retrieval_failure(self):
@@ -255,9 +213,9 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
             rv.read.return_value = 'XXXXX'
             self.assertEqual([], sp.servers[0].get_capabilities())
 
-            # One broken server should affect all capabilities
+            # as capabilities is empty, it should try to update capabilities
             rv.read.side_effect = ['{"a": "b"}', '["b","c","d"]']
-            self.assertEqual(set(), sp.get_capabilities())
+            self.assertEqual(set(['a', 'b', 'c', 'd']), sp.get_capabilities())
 
     def test_reconnect_on_timeout_change(self):
         sp = servermanager.ServerPool()
@@ -402,11 +360,11 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
         with mock.patch(SERVERMANAGER + '.ServerProxy.rest_call',
                         return_value=(httplib.SERVICE_UNAVAILABLE,
                                       0, 0, 0)) as srestmock,\
-                mock.patch(SERVERMANAGER + '.time.sleep') as tmock:
+                mock.patch(SERVERMANAGER + '.eventlet.sleep') as tmock:
             # making a call should trigger retries with sleeps in between
             pl.servers.rest_call('GET', '/', '', None, [])
-            rest_call = [mock.call('GET', '/', '', None, False, reconnect=True,
-                                   hash_handler=mock.ANY)]
+            rest_call = [mock.call('GET', '/', '', None, False,
+                                   reconnect=True)]
             rest_call_count = (
                 servermanager.HTTP_SERVICE_UNAVAILABLE_RETRY_COUNT + 1)
             srestmock.assert_has_calls(rest_call * rest_call_count)
@@ -416,52 +374,74 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
             sleep_call_count = rest_call_count - 1
             tmock.assert_has_calls(sleep_call * sleep_call_count)
 
-    def test_delete_failure_sets_bad_hash(self):
+    def test_delete_failure_forces_topo_sync(self):
         pl = directory.get_plugin()
-        hash_handler = consistency_db.HashHandler()
-        with mock.patch(
-            SERVERMANAGER + '.ServerProxy.rest_call',
-            return_value=(httplib.INTERNAL_SERVER_ERROR, 0, 0, 0)
-        ):
-            # a failed delete call should put a bad hash in the DB
-            pl.servers.rest_call('DELETE', '/', '', None, [])
-            self.assertEqual('INCONSISTENT,INCONSISTENT',
-                             hash_handler.read_for_update())
+        with mock.patch(SERVERMANAGER + '.ServerProxy.rest_call',
+                        return_value=(httplib.INTERNAL_SERVER_ERROR,
+                                      0, 0, 0)), \
+                mock.patch(SERVERMANAGER + '.ServerPool.force_topo_sync',
+                           return_value=(False,
+                                         servermanager.TOPO_RESPONSE_OK)) \
+                as topo_mock:
+            # a failed DELETE call should trigger a forced topo_sync
+            # with check_ts True
+            self.assertRaises(servermanager.RemoteRestError,
+                              pl.servers.rest_action,
+                              **{'action': 'DELETE', 'resource': '/',
+                                 'data': '',
+                                 'errstr': "Unable to DELETE query to BCF: %s",
+                                 'ignore_codes': []})
+            topo_mock.assert_called_once_with(**{'check_ts': True})
 
-    def test_conflict_triggers_sync(self):
+    def test_post_failure_forces_topo_sync(self):
         pl = directory.get_plugin()
+        with mock.patch(SERVERMANAGER + '.ServerProxy.rest_call',
+                        return_value=(httplib.INTERNAL_SERVER_ERROR,
+                                      0, 0, 0)), \
+                mock.patch(SERVERMANAGER + '.ServerPool.force_topo_sync',
+                           return_value=(False,
+                                         servermanager.TOPO_RESPONSE_OK)) \
+                as topo_mock:
+            # a failed POST call should trigger a forced topo_sync
+            # with check_ts True
+            self.assertRaises(servermanager.RemoteRestError,
+                              pl.servers.rest_action,
+                              **{'action': 'POST', 'resource': '/', 'data': '',
+                                 'errstr': "Unable to POST query to BCF: %s",
+                                 'ignore_codes': []})
+            topo_mock.assert_called_once_with(**{'check_ts': True})
+
+    def test_topo_sync_failure_does_not_force_topo_sync(self):
+        pl = directory.get_plugin()
+        with mock.patch(SERVERMANAGER + '.ServerProxy.rest_call',
+                        return_value=(httplib.INTERNAL_SERVER_ERROR,
+                                      0, 0, 0)), \
+                mock.patch(SERVERMANAGER + '.ServerPool.force_topo_sync',
+                           return_value=(False,
+                                         servermanager.TOPO_RESPONSE_OK)) \
+                as topo_mock:
+            # a failed POST call for topology path should raise an exception
+            # and not call force_topo_sync like other failed rest_action
+            self.assertRaises(servermanager.RemoteRestError,
+                              pl.servers.rest_action,
+                              **{'action': 'POST', 'resource': '/topology',
+                                 'data': '',
+                                 'errstr': "Unable to perform topo_sync: %s",
+                                 'ignore_codes': []})
+            topo_mock.assert_not_called()
+
+    def test_not_found_sync_raises_error_without_topology(self):
+        pl = directory.get_plugin()
+        pl.servers.get_topo_function = None
         with \
             mock.patch(
                 SERVERMANAGER + '.ServerProxy.rest_call',
-                return_value=(httplib.CONFLICT, 0, 0, 0)) as srestmock, \
-            mock.patch(
-                CONSISTENCYDB + '.HashHandler.is_db_lock_owner',
-                return_value=True):
-            # making a call should trigger a conflict sync
-            pl.servers.rest_call('GET', '/', '', None, [])
-            srestmock.assert_has_calls([
-                mock.call('GET', '/', '', None, False, reconnect=True,
-                          hash_handler=mock.ANY),
-                mock.call('POST', '/topology',
-                          {'routers': [], 'security-groups': [],
-                           'networks': [],
-                           'tenants': {'VRRP_Service': 'VRRP_Service'}},
-                          hash_handler=mock.ANY, timeout=None)
-            ])
-
-    def test_conflict_sync_raises_error_without_topology(self):
-        pl = directory.get_plugin()
-        pl.servers.get_topo_function = None
-        with mock.patch(SERVERMANAGER + '.ServerProxy.rest_call',
-                        return_value=(httplib.CONFLICT, 0, 0, 0)), \
-                mock.patch(
-                    CONSISTENCYDB + '.HashHandler.is_db_lock_owner',
-                    return_value=True,):
+                return_value=(httplib.NOT_FOUND, 0, 0, 0)):
             # making a call should trigger a conflict sync that will
             # error without the topology function set
             self.assertRaises(
                 cfg.Error,
-                pl.servers.rest_call,
+                pl.servers.rest_action,
                 *('GET', '/', '', None, [])
             )
 
@@ -475,15 +455,18 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
             # making a call should trigger a conflict sync
             pl.servers.rest_call('GET', '/', '', None, [])
             srestmock.assert_called_once_with(
-                'GET', '/', '', None, False, reconnect=True,
-                hash_handler=mock.ANY)
+                'GET', '/', '', None, False, reconnect=True)
 
     def test_no_send_all_data_without_keystone(self):
         pl = directory.get_plugin()
         with mock.patch(SERVERMANAGER + '.ServerPool._update_tenant_cache',
-                        return_value=(False)):
+                        return_value=(False)), \
+            mock.patch(SERVERMANAGER + '.ServerPool.force_topo_sync',
+                       return_value=(False, servermanager.TOPO_RESPONSE_OK)) \
+                as tmock:
             # making a call should trigger a conflict sync
-            self.assertIsNone(pl._send_all_data())
+            self.assertRaises(Exception, pl._send_all_data())  # noqa
+            tmock.assert_called_once()
 
     def test_floating_calls(self):
         pl = directory.get_plugin()
@@ -514,7 +497,7 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
         )])
         self.wrap_mock.assert_has_calls([mock.call(
             self.socket_mock(), None, None, cert_reqs=ssl.CERT_NONE,
-            ssl_version=ssl.PROTOCOL_TLSv1
+            ssl_version=ssl.PROTOCOL_SSLv23
         )])
         self.assertEqual(con.sock, self.wrap_mock())
 
@@ -530,7 +513,7 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
         self.wrap_mock.assert_has_calls([mock.call(
             self.socket_mock(), None, None, ca_certs='SOMECERTS.pem',
             cert_reqs=ssl.CERT_REQUIRED,
-            ssl_version=ssl.PROTOCOL_TLSv1
+            ssl_version=ssl.PROTOCOL_SSLv23
         )])
         self.assertEqual(con.sock, self.wrap_mock())
 
@@ -548,7 +531,7 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
         )])
         self.wrap_mock.assert_has_calls([mock.call(
             self.socket_mock(), None, None, cert_reqs=ssl.CERT_NONE,
-            ssl_version=ssl.PROTOCOL_TLSv1
+            ssl_version=ssl.PROTOCOL_SSLv23
         )])
         # _tunnel() doesn't take any args
         tunnel_mock.assert_has_calls([mock.call()])
@@ -556,6 +539,59 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
         self.assertEqual(con._tunnel_port, 3128)
         self.assertEqual(con.sock, self.wrap_mock())
 
+    def test_is_unicode_enabled(self):
+        """Verify that unicode is enabled only when both conditions are True:
+
+         1. naming_scheme_unicode is True or empty
+         2. BCF capabilities include display-name
+
+        :return:
+        """
+        self.is_unicode_enabled_p.stop()
+
+        def capability_unicode_supported():
+            return ['dummy', 'display-name']
+
+        def capability_unicode_unsupported():
+            return ['dummy']
+
+        patch_supported = mock.patch(
+            SERVER_GET_CAPABILITIES,
+            side_effect=capability_unicode_supported)
+
+        patch_unsupported = mock.patch(
+            SERVER_GET_CAPABILITIES,
+            side_effect=capability_unicode_unsupported)
+
+        # Create a server pool with default naming_scheme_unicode
+        # verify default value is true
+        sp = servermanager.ServerPool()
+        self.assertTrue(cfg.CONF.RESTPROXY.naming_scheme_unicode)
+
+        # config enabled, and unicode is supported on bcf
+        patch_supported.start()
+        self.assertTrue(sp.is_unicode_enabled())
+        patch_supported.stop()
+
+        # config enabled, but unicode is not supported on bcf
+        patch_unsupported.start()
+        self.assertFalse(sp.is_unicode_enabled())
+        patch_unsupported.stop()
+
+        # Recreate the server pool, as the config is read during initialization
+        cfg.CONF.set_override('naming_scheme_unicode', False, 'RESTPROXY')
+        sp = servermanager.ServerPool()
+
+        # config disabled, though unicode is supported on bcf
+        patch_supported.start()
+        self.assertFalse(sp.is_unicode_enabled())
+        patch_supported.stop()
+
+        # config disabled, and unicode is not supported on bcf
+        patch_unsupported.start()
+        self.assertFalse(sp.is_unicode_enabled())
+        patch_unsupported.stop()
+
 
 class TestSockets(test_rp.BigSwitchProxyPluginV2TestCase):
 
@@ -582,45 +618,144 @@ class HashLockingTests(test_rp.BigSwitchProxyPluginV2TestCase):
                    filter_by(hash_id=handler.hash_id).first())
             return res.hash
 
-    def test_hash_handle_lock_no_initial_record(self):
+    def test_lock_no_initial_record(self):
         handler = consistency_db.HashHandler()
-        h1 = handler.read_for_update()
-        # return to caller should be empty even with lock in DB
-        self.assertFalse(h1)
+        h1 = handler.lock()
+        # lock() request on empty DB should succeed
+        self.assertTrue(h1)
         # db should have a lock marker
         self.assertEqual(handler.lock_marker,
                          self._get_hash_from_handler_db(handler))
-        # an entry should clear the lock
-        handler.put_hash('DIGEST')
-        self.assertEqual('DIGEST', self._get_hash_from_handler_db(handler))
-
-    def test_hash_handle_lock_existing_record(self):
-        handler = consistency_db.HashHandler()
-        handler.put_hash('DIGEST')  # set initial hash
-
-        h1 = handler.read_for_update()
-        self.assertEqual('DIGEST', h1)
-        self.assertEqual(handler.lock_marker + 'DIGEST',
+        # prev_lock_ts must be 0 for initial case
+        self.assertEqual(handler.prev_lock_ts, '0')
+        # unlock() should clear the lock
+        handler.unlock()
+        self.assertEqual(handler.lock_ts,
                          self._get_hash_from_handler_db(handler))
 
-        # make sure update works
-        handler.put_hash('DIGEST2')
-        self.assertEqual('DIGEST2', self._get_hash_from_handler_db(handler))
-
     def test_db_duplicate_on_insert(self):
         handler = consistency_db.HashHandler()
         with mock.patch.object(
             handler.session, 'add', side_effect=[db_exc.DBDuplicateEntry, '']
         ) as add_mock:
-            handler.read_for_update()
+            handler.lock()
             # duplicate insert failure should result in retry
             self.assertEqual(2, add_mock.call_count)
 
-    def test_update_hit_no_records(self):
-        handler = consistency_db.HashHandler()
-        # set initial hash so update will be required
-        handler.put_hash('DIGEST')
-        with mock.patch.object(handler._FACADE, 'get_engine') as ge:
+    def test_lock_check_ts_true_prev_lock_exists(self):
+        handler1 = consistency_db.HashHandler()
+        h1 = handler1.lock()
+        self.assertTrue(h1)
+        self.assertEqual(handler1.lock_marker,
+                         self._get_hash_from_handler_db(handler1))
+
+        # 2nd thread came in just 10 millisecs after first one, and first one
+        # still holds the lock, expired  = False
+        timestamp_2 = float(handler1.lock_ts) + 10
+        handler2 = consistency_db.HashHandler(timestamp_ms=timestamp_2)
+        h2 = handler2.lock()
+        self.assertFalse(h2)
+        self.assertEqual(handler1.lock_ts, handler2.prev_lock_ts)
+
+    def test_lock_check_ts_false_prev_lock_exists(self):
+        handler1 = consistency_db.HashHandler()
+        h1 = handler1.lock()
+        self.assertTrue(h1)
+        self.assertEqual(handler1.lock_marker,
+                         self._get_hash_from_handler_db(handler1))
+        self.assertEqual('0', handler1.prev_lock_ts)
+
+        hh2_ts_hh1_ts_plus_1780 = float(handler1.lock_ts) + 1780
+        handler2 = consistency_db.HashHandler(
+            hash_id='1', timestamp_ms=hh2_ts_hh1_ts_plus_1780)
+        with mock.patch(CONSISTENCYDB + '.eventlet.sleep',
+                        side_effect=[Exception]) as emock:
+            try:
+                handler2.lock(check_ts=False)
+            except Exception:
+                pass
+        self.assertEqual(1, emock.call_count)
+        self.assertEqual(handler1.lock_ts, handler2.prev_lock_ts)
+
+    def test_lock_check_ts_true_prev_lock_not_expired(self):
+        handler1 = consistency_db.HashHandler()
+        h1 = handler1.lock()
+        self.assertTrue(h1)
+        self.assertEqual(handler1.lock_marker,
+                         self._get_hash_from_handler_db(handler1))
+
+        handler1.unlock()
+        self.assertEqual(handler1.lock_ts,
+                         self._get_hash_from_handler_db(handler1))
+
+        # thread 1 has executed the complete lock-unlock cycle
+        # thread 2 now tries to get lock with check_ts True
+        # TOPO_SYNC_EXPIRED_SECS = 1800
+        hh2_ts_under_limit = float(handler1.lock_ts) + 1000
+        handler2 = consistency_db.HashHandler(hash_id=1,
+                                              timestamp_ms=hh2_ts_under_limit)
+        h2 = handler2.lock()
+        self.assertFalse(h2)
+        self.assertEqual(handler1.lock_ts, handler2.prev_lock_ts)
+
+    def test_lock_check_ts_true_prev_lock_expired(self):
+        handler1 = consistency_db.HashHandler()
+        h1 = handler1.lock()
+        self.assertTrue(h1)
+        self.assertEqual(handler1.lock_marker,
+                         self._get_hash_from_handler_db(handler1))
+
+        handler1.unlock()
+        self.assertEqual(handler1.lock_ts,
+                         self._get_hash_from_handler_db(handler1))
+
+        # thread 1 has executed the complete lock-unlock cycle
+        # thread 2 now tries to get lock with check_ts True
+        # TOPO_SYNC_EXPIRED_SECS = 1 for testing
+        time.sleep(1)
+        handler2 = consistency_db.HashHandler()
+        # only for testing
+        consistency_db.TOPO_SYNC_EXPIRED_SECS = 1
+        h2 = handler2.lock()
+        self.assertTrue(h2)
+        self.assertEqual(handler1.lock_ts, handler2.prev_lock_ts)
+
+    def test_lock_check_ts_false_prev_lock_not_expired(self):
+        handler1 = consistency_db.HashHandler()
+        h1 = handler1.lock()
+        self.assertTrue(h1)
+        self.assertEqual(handler1.lock_marker,
+                         self._get_hash_from_handler_db(handler1))
+
+        handler1.unlock()
+        self.assertEqual(handler1.lock_ts,
+                         self._get_hash_from_handler_db(handler1))
+
+        # thread 1 has executed the complete lock-unlock cycle
+        # thread 2 now tries to get lock with check_ts True
+        # TOPO_SYNC_EXPIRED_SECS = 1800
+        hh2_ts_under_limit = float(handler1.lock_ts) + 1000
+        handler2 = consistency_db.HashHandler(hash_id=1,
+                                              timestamp_ms=hh2_ts_under_limit)
+        h2 = handler2.lock(check_ts=False)
+        self.assertTrue(h2)
+        self.assertEqual(handler1.lock_ts, handler2.prev_lock_ts)
+
+    def test_lock_check_ts_false_lock_clash(self):
+        # 2 threads try to lock the DB at the same time when check_ts is False
+        # and no thread holds the lock
+        handler1 = consistency_db.HashHandler()
+        h1 = handler1.lock()
+        self.assertTrue(h1)
+        handler1.unlock()
+        self.assertEqual(handler1.lock_ts,
+                         self._get_hash_from_handler_db(handler1))
+
+        handler2 = consistency_db.HashHandler()
+
+        with mock.patch.object(handler2._FACADE, 'get_engine') as ge, \
+                mock.patch(CONSISTENCYDB + '.eventlet.sleep',
+                           side_effect=[None]) as emock:
             conn = ge.return_value.begin.return_value.__enter__.return_value
             firstresult = mock.Mock()
             # a rowcount of 0 simulates the effect of another db client
@@ -629,68 +764,55 @@ class HashLockingTests(test_rp.BigSwitchProxyPluginV2TestCase):
             secondresult = mock.Mock()
             secondresult.rowcount = 1
             conn.execute.side_effect = [firstresult, secondresult]
-            handler.read_for_update()
+            h2 = handler2.lock(check_ts=False)
+            self.assertTrue(h2)
             # update should have been called again after the failure
             self.assertEqual(2, conn.execute.call_count)
-
-    def test_handler_already_holding_lock(self):
-        handler = consistency_db.HashHandler()
-        handler.read_for_update()  # lock the table
-        with mock.patch.object(handler._FACADE, 'get_engine') as ge:
-            handler.read_for_update()
-            # get engine should not have been called because no update
-            # should have been made
-            self.assertFalse(ge.called)
+            # sleep should have been called once, during first result failure
+            emock.assert_called_once()
 
     def test_clear_lock(self):
         handler = consistency_db.HashHandler()
-        handler.put_hash('SOMEHASH')
-        handler.read_for_update()  # lock the table
-        self.assertEqual(handler.lock_marker + 'SOMEHASH',
+        handler.lock()  # lock the table
+        self.assertEqual(handler.lock_marker,
                          self._get_hash_from_handler_db(handler))
-        handler.clear_lock()
-        self.assertEqual('SOMEHASH',
+        handler.unlock()
+        self.assertEqual(handler.lock_ts,
                          self._get_hash_from_handler_db(handler))
 
-    def test_clear_lock_skip_after_steal(self):
-        handler1 = consistency_db.HashHandler()
-        handler1.read_for_update()  # lock the table
-        handler2 = consistency_db.HashHandler()
-        with mock.patch.object(consistency_db, 'MAX_LOCK_WAIT_TIME', new=0):
-            handler2.read_for_update()
-            before = self._get_hash_from_handler_db(handler1)
-            # handler1 should not clear handler2's lock
-            handler1.clear_lock()
-            self.assertEqual(before, self._get_hash_from_handler_db(handler1))
+    def test_handler_already_holding_lock(self):
+        handler = consistency_db.HashHandler()
+        handler.lock()  # lock the table
+        with mock.patch.object(handler._FACADE, 'get_engine') as ge:
+            handler.lock()
+            # get engine should not have been called because no update
+            # should have been made
+            self.assertFalse(ge.called)
+            self.assertTrue(handler.lock_ts, handler.prev_lock_ts)
 
-    def test_take_lock_from_other(self):
+    def test_unlock_set_prev_ts(self):
         handler1 = consistency_db.HashHandler()
-        handler1.read_for_update()  # lock the table
-        handler2 = consistency_db.HashHandler()
-        with mock.patch.object(consistency_db, 'MAX_LOCK_WAIT_TIME') as mlock:
-            # make handler2 wait for only one iteration
-            mlock.__lt__.side_effect = [False, True]
-            handler2.read_for_update()
-            # once MAX LOCK exceeded, comparisons should stop due to lock steal
-            self.assertEqual(2, mlock.__lt__.call_count)
-            dbentry = self._get_hash_from_handler_db(handler1)
-            # handler2 should have the lock
-            self.assertIn(handler2.lock_marker, dbentry)
-            self.assertNotIn(handler1.lock_marker, dbentry)
-            # lock protection only blocks read_for_update, anyone can change
-            handler1.put_hash('H1')
+        handler1.lock()
+        self.assertEqual(handler1.lock_marker,
+                         self._get_hash_from_handler_db(handler1))
+        handler1.unlock()
 
-    def test_failure_to_steal_lock(self):
-        handler1 = consistency_db.HashHandler()
-        handler1.read_for_update()  # lock the table
+        # first lock-unlock is done. now comes a second call with
+        # check_ts = False
         handler2 = consistency_db.HashHandler()
-        with\
-            mock.patch.object(consistency_db, 'MAX_LOCK_WAIT_TIME') as mlock,\
-            mock.patch.object(handler2, '_optimistic_update_hash_record',
-                              side_effect=[False, True]) as oplock:
-            # handler2 will go through 2 iterations since the lock will fail on
-            # the first attempt
-            mlock.__lt__.side_effect = [False, True, False, True]
-            handler2.read_for_update()
-            self.assertEqual(4, mlock.__lt__.call_count)
-            self.assertEqual(2, oplock.call_count)
+        h2 = handler2.lock(check_ts=False)
+        self.assertTrue(h2)
+        self.assertEqual(handler1.lock_ts, handler2.prev_lock_ts)
+
+        # now assuming exception occured during topo_sync, call
+        # handler2.unlock(set_prev_ts=True)
+        handler2.unlock(set_prev_ts=True)
+        # hash in consistency_db will be previous hash_handler's lock_ts
+        self.assertEqual(handler1.lock_ts,
+                         self._get_hash_from_handler_db(handler2))
+        # try unlock again on the same handler2 - it should have no effect
+        # as unlock(set_prev_ts) removed TOPO_SYNC marker. this simulates
+        # unlock() being called in the finally block of force_topo_sync()
+        handler2.unlock()
+        self.assertEqual(handler1.lock_ts,
+                         self._get_hash_from_handler_db(handler2))
